@@ -25,13 +25,12 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
-import java.util.zip.Deflater;
-import java.util.zip.DeflaterOutputStream;
-import java.util.zip.InflaterInputStream;
+import java.util.Objects;
 
-import com.google.common.base.Preconditions;
 import com.google.protobuf.ByteString;
 
+import com.google.protobuf.CodedInputStream;
+import org.apache.tez.runtime.api.TaskContext;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.apache.hadoop.classification.InterfaceAudience;
@@ -42,6 +41,8 @@ import org.apache.tez.dag.api.UserPayload;
 import org.apache.tez.dag.api.records.DAGProtos;
 import org.codehaus.jettison.json.JSONException;
 import org.codehaus.jettison.json.JSONObject;
+import org.xerial.snappy.SnappyInputStream;
+import org.xerial.snappy.SnappyOutputStream;
 
 /**
  * Utility methods for setting up a DAG. Has helpers for setting up log4j configuration, converting
@@ -75,10 +76,9 @@ public class TezUtils {
    * @throws java.io.IOException
    */
   public static ByteString createByteStringFromConf(Configuration conf) throws IOException {
-    Preconditions.checkNotNull(conf, "Configuration must be specified");
+    Objects.requireNonNull(conf, "Configuration must be specified");
     ByteString.Output os = ByteString.newOutput();
-    DeflaterOutputStream compressOs = new DeflaterOutputStream(os,
-        new Deflater(Deflater.BEST_SPEED));
+    SnappyOutputStream compressOs = new SnappyOutputStream(os);
     try {
       writeConfInPB(compressOs, conf);
     } finally {
@@ -101,6 +101,12 @@ public class TezUtils {
     return UserPayload.create(ByteBuffer.wrap(createByteStringFromConf(conf).toByteArray()));
   }
 
+  private static DAGProtos.ConfigurationProto createConfProto(SnappyInputStream uncompressIs) throws IOException {
+    CodedInputStream in = CodedInputStream.newInstance(uncompressIs);
+    in.setSizeLimit(Integer.MAX_VALUE);
+    return DAGProtos.ConfigurationProto.parseFrom(in);
+  }
+
   /**
    * Convert a byte string to a Configuration object
    *
@@ -110,17 +116,39 @@ public class TezUtils {
    * @throws java.io.IOException
    */
   public static Configuration createConfFromByteString(ByteString byteString) throws IOException {
-    Preconditions.checkNotNull(byteString, "ByteString must be specified");
-    // SnappyInputStream uncompressIs = new
-    // SnappyInputStream(byteString.newInput());
-    try(InflaterInputStream uncompressIs = new InflaterInputStream(byteString.newInput())) {
-      DAGProtos.ConfigurationProto confProto = DAGProtos.ConfigurationProto.parseFrom(uncompressIs);
+    Objects.requireNonNull(byteString, "ByteString must be specified");
+    try(SnappyInputStream uncompressIs = new SnappyInputStream(byteString.newInput());) {
+      DAGProtos.ConfigurationProto confProto = createConfProto(uncompressIs);
       Configuration conf = new Configuration(false);
       readConfFromPB(confProto, conf);
+      TezClassLoader.setupForConfiguration(conf);
       return conf;
     }
   }
 
+  public static Configuration createConfFromBaseConfAndPayload(TaskContext context)
+      throws IOException {
+    Configuration baseConf = context.getContainerConfiguration();
+    Configuration configuration = new Configuration(baseConf);
+    UserPayload payload = context.getUserPayload();
+    ByteString byteString = ByteString.copyFrom(payload.getPayload());
+    try(SnappyInputStream uncompressIs = new SnappyInputStream(byteString.newInput())) {
+      DAGProtos.ConfigurationProto confProto = createConfProto(uncompressIs);
+      readConfFromPB(confProto, configuration);
+      TezClassLoader.setupForConfiguration(configuration);
+      return configuration;
+    }
+  }
+
+  public static void addToConfFromByteString(Configuration configuration, ByteString byteString)
+      throws IOException {
+    try(SnappyInputStream uncompressIs = new SnappyInputStream(byteString.newInput())) {
+      DAGProtos.ConfigurationProto confProto = createConfProto(uncompressIs);
+      readConfFromPB(confProto, configuration);
+      TezClassLoader.setupForConfiguration(configuration);
+    }
+  }
+
   /**
    * Convert an instance of {@link org.apache.tez.dag.api.UserPayload} to {@link
    * org.apache.hadoop.conf.Configuration}
