@@ -29,7 +29,8 @@ import java.util.Map;
 import java.util.Set;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Preconditions;
+import org.apache.hadoop.security.UserGroupInformation;
+import org.apache.tez.common.Preconditions;
 
 import org.apache.hadoop.yarn.exceptions.ApplicationNotFoundException;
 import org.slf4j.Logger;
@@ -39,7 +40,6 @@ import org.apache.hadoop.yarn.api.records.ApplicationId;
 import org.apache.hadoop.yarn.api.records.ApplicationReport;
 import org.apache.hadoop.yarn.api.records.FinalApplicationStatus;
 import org.apache.hadoop.yarn.api.records.YarnApplicationState;
-import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.exceptions.YarnException;
 import org.apache.tez.client.FrameworkClient;
 import org.apache.tez.common.counters.TezCounters;
@@ -79,7 +79,7 @@ public class DAGClientImpl extends DAGClient {
   private boolean cleanupFrameworkClient;
 
   public DAGClientImpl(ApplicationId appId, String dagId, TezConfiguration conf,
-      YarnConfiguration yarnConf, @Nullable FrameworkClient frameworkClient) {
+      @Nullable FrameworkClient frameworkClient, UserGroupInformation ugi) {
     this.appId = appId;
     this.dagId = dagId;
     this.conf = conf;
@@ -87,7 +87,7 @@ public class DAGClientImpl extends DAGClient {
       this.frameworkClient = frameworkClient;
     } else {
       this.frameworkClient = FrameworkClient.createFrameworkClient(conf);
-      this.frameworkClient.init(conf, yarnConf);
+      this.frameworkClient.init(conf);
       this.frameworkClient.start();
       cleanupFrameworkClient = true;
     }
@@ -99,7 +99,7 @@ public class DAGClientImpl extends DAGClient {
             TezConfiguration.TEZ_AM_HISTORY_LOGGING_ENABLED_DEFAULT) &&
         DAGClientTimelineImpl.isSupported();
 
-    realClient = new DAGClientRPCImpl(appId, dagId, conf, this.frameworkClient);
+    realClient = new DAGClientRPCImpl(appId, dagId, conf, this.frameworkClient, ugi);
     statusPollInterval = conf.getLong(
         TezConfiguration.TEZ_DAG_STATUS_POLLINTERVAL_MS,
         TezConfiguration.TEZ_DAG_STATUS_POLLINTERVAL_MS_DEFAULT);
@@ -209,7 +209,7 @@ public class DAGClientImpl extends DAGClient {
     }
   }
 
-  private DAGStatus getDAGStatusInternal(@Nullable Set<StatusGetOpts> statusOptions,
+  protected DAGStatus getDAGStatusInternal(@Nullable Set<StatusGetOpts> statusOptions,
       long timeout) throws TezException, IOException {
 
     if (!dagCompleted) {
@@ -247,9 +247,7 @@ public class DAGClientImpl extends DAGClient {
         LOG.info("Failed to fetch DAG data for completed DAG from YARN Timeline"
             + " - Application not found by YARN", e);
       } catch (TezException e) {
-        if (LOG.isDebugEnabled()) {
-          LOG.info("DAGStatus fetch failed." + e.getMessage());
-        }
+        LOG.debug("DAGStatus fetch failed", e);
       }
     }
 
@@ -302,9 +300,7 @@ public class DAGClientImpl extends DAGClient {
             + " - Application not found by YARN", e);
         return null;
       } catch (TezException e) {
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("ERROR fetching vertex data from Yarn Timeline. " + e.getMessage());
-        }
+        LOG.debug("ERROR fetching vertex data from Yarn Timeline", e);
       }
     }
 
@@ -425,9 +421,7 @@ public class DAGClientImpl extends DAGClient {
    */
   @VisibleForTesting
   protected DAGStatus getDAGStatusViaRM() throws TezException, IOException {
-    if(LOG.isDebugEnabled()) {
-      LOG.debug("GetDAGStatus via AM for app: " + appId + " dag:" + dagId);
-    }
+    LOG.debug("GetDAGStatus via AM for app: {} dag:{}", appId, dagId);
     ApplicationReport appReport;
     try {
       appReport = frameworkClient.getApplicationReport(appId);
@@ -638,9 +632,7 @@ public class DAGClientImpl extends DAGClient {
     realClient.close();
     realClient = new DAGClientTimelineImpl(appId, dagId, conf, frameworkClient,
         (int) (2 * PRINT_STATUS_INTERVAL_MILLIS));
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("dag completed switching to DAGClientTimelineImpl");
-    }
+    LOG.debug("dag completed switching to DAGClientTimelineImpl");
   }
 
   @VisibleForTesting
