@@ -23,6 +23,7 @@ import java.util.Set;
 
 import javax.annotation.Nullable;
 
+import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.yarn.exceptions.ApplicationNotFoundException;
 import org.apache.tez.common.RPCUtil;
 import org.apache.tez.dag.api.SessionNotRunning;
@@ -68,13 +69,16 @@ public class DAGClientRPCImpl extends DAGClientInternal {
   @VisibleForTesting
   DAGClientAMProtocolBlockingPB proxy = null;
 
+  private UserGroupInformation ugi;
+
   public DAGClientRPCImpl(ApplicationId appId, String dagId,
-      TezConfiguration conf, @Nullable FrameworkClient frameworkClient) {
+      TezConfiguration conf, @Nullable FrameworkClient frameworkClient, UserGroupInformation ugi) {
     this.appId = appId;
     this.dagId = dagId;
     this.conf = conf;
     this.frameworkClient = frameworkClient;
     appReport = null;
+    this.ugi = ugi;
   }
 
   @Override
@@ -142,9 +146,7 @@ public class DAGClientRPCImpl extends DAGClientInternal {
 
   @Override
   public void tryKillDAG() throws TezException, IOException {
-    if(LOG.isDebugEnabled()) {
-      LOG.debug("TryKill for app: " + appId + " dag:" + dagId);
-    }
+    LOG.debug("TryKill for app: {} dag:{}", appId, dagId);
     try {
       if (createAMProxyIfNeeded()) {
         TryKillDAGRequestProto requestProto =
@@ -182,9 +184,7 @@ public class DAGClientRPCImpl extends DAGClientInternal {
 
   DAGStatus getDAGStatusViaAM(Set<StatusGetOpts> statusOptions, long timeout)
       throws IOException, TezException {
-    if(LOG.isDebugEnabled()) {
-      LOG.debug("GetDAGStatus via AM for app: " + appId + " dag:" + dagId);
-    }
+    LOG.debug("GetDAGStatus via AM for app: {} dag:{}", appId, dagId);
     GetDAGStatusRequestProto.Builder requestProtoBuilder =
         GetDAGStatusRequestProto.newBuilder()
           .setDagId(dagId).setTimeout(timeout);
@@ -286,7 +286,7 @@ public class DAGClientRPCImpl extends DAGClientInternal {
     }
 
     proxy = TezClientUtils.getAMProxy(conf, appReport.getHost(), appReport.getRpcPort(),
-        appReport.getClientToAMToken());
+        appReport.getClientToAMToken(), ugi);
     return true;
   }
 
