@@ -18,6 +18,7 @@
 
 package org.apache.tez.client;
 
+import java.io.File;
 import java.io.IOException;
 import java.net.InetAddress;
 import java.nio.ByteBuffer;
@@ -55,6 +56,7 @@ import org.apache.hadoop.fs.CommonConfigurationKeys;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.security.Credentials;
+import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.util.Time;
 import org.apache.hadoop.yarn.api.records.ApplicationId;
 import org.apache.hadoop.yarn.api.records.ApplicationReport;
@@ -66,7 +68,6 @@ import org.apache.hadoop.yarn.api.records.Resource;
 import org.apache.hadoop.yarn.api.records.URL;
 import org.apache.hadoop.yarn.api.records.YarnApplicationState;
 import org.apache.hadoop.yarn.client.api.YarnClient;
-import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.exceptions.ApplicationNotFoundException;
 import org.apache.hadoop.yarn.exceptions.YarnException;
 import org.apache.tez.common.counters.LimitExceededException;
@@ -93,7 +94,6 @@ import org.apache.tez.dag.api.client.rpc.DAGClientAMProtocolRPC.GetDAGStatusResp
 import org.apache.tez.dag.api.client.rpc.DAGClientAMProtocolRPC.ShutdownSessionRequestProto;
 import org.apache.tez.dag.api.client.rpc.DAGClientAMProtocolRPC.SubmitDAGRequestProto;
 import org.apache.tez.dag.api.client.rpc.DAGClientAMProtocolRPC.TezAppMasterStatusProto;
-import org.apache.tez.dag.api.records.DAGProtos.ConfigurationProto;
 import org.apache.tez.dag.api.records.DAGProtos.DAGStatusProto;
 import org.apache.tez.dag.api.records.DAGProtos.DAGStatusStateProto;
 import org.apache.tez.dag.api.records.DAGProtos.ProgressProto;
@@ -108,9 +108,10 @@ import com.google.protobuf.RpcController;
 
 public class TestTezClient {
   static final long HARD_KILL_TIMEOUT = 1500L;
+  private static final File STAGING_DIR = new File(System.getProperty("test.build.data"),
+      TestTezClient.class.getName()).getAbsoluteFile();
 
   class TezClientForTest extends TezClient {
-    TezYarnClient mockTezYarnClient;
     DAGClientAMProtocolBlockingPB sessionAmProxy;
     YarnClient mockYarnClient;
     ApplicationId mockAppId;
@@ -118,23 +119,13 @@ public class TestTezClient {
     Long prewarmTimeoutMs;
 
     public TezClientForTest(String name, TezConfiguration tezConf,
-        @Nullable Map<String, LocalResource> localResources,
-        @Nullable Credentials credentials) {
+        @Nullable Map<String, LocalResource> localResources, @Nullable Credentials credentials) {
       super(name, tezConf, localResources, credentials);
     }
-    
+
     @Override
     protected FrameworkClient createFrameworkClient() {
-      return mockTezYarnClient;
-    }
-    
-    @Override
-    protected DAGClientAMProtocolBlockingPB getAMProxy(ApplicationId appId)
-        throws TezException, IOException {
-      if (!callRealGetSessionAMProxy) {
-        return sessionAmProxy;
-      }
-      return super.getAMProxy(appId);
+      return frameworkClient; // already initialized
     }
 
     public void setPrewarmTimeoutMs(Long prewarmTimeoutMs) {
@@ -146,7 +137,34 @@ public class TestTezClient {
       return prewarmTimeoutMs == null ? super.getPrewarmWaitTimeMs() : prewarmTimeoutMs;
     }
   }
-  
+
+  class TezYarnClientForTest extends TezYarnClient {
+    private TezClientForTest client;
+
+    protected TezYarnClientForTest(YarnClient yarnClient, TezClientForTest client) {
+      super(yarnClient);
+      this.client = client;
+    }
+
+    @Override
+    protected DAGClientAMProtocolBlockingPB waitForProxy(long clientTimeout, Configuration conf,
+        ApplicationId sessionAppId, UserGroupInformation ugi) throws TezException, IOException {
+      if (!client.callRealGetSessionAMProxy) {
+        return client.sessionAmProxy;
+      }
+      return super.getProxy(conf, sessionAppId, ugi);
+    }
+
+    @Override
+    protected DAGClientAMProtocolBlockingPB getProxy(Configuration conf, ApplicationId sessionAppId,
+        UserGroupInformation ugi) throws TezException, IOException {
+      if (!client.callRealGetSessionAMProxy) {
+        return client.sessionAmProxy;
+      }
+      return super.getProxy(conf, sessionAppId, ugi);
+    }
+  }
+
   TezClientForTest configureAndCreateTezClient() throws YarnException, IOException, ServiceException {
     return configureAndCreateTezClient(null);
   }
@@ -163,6 +181,7 @@ public class TestTezClient {
     }
     conf.setBoolean(TezConfiguration.TEZ_IGNORE_LIB_URIS, true);
     conf.setBoolean(TezConfiguration.TEZ_AM_SESSION_MODE, isSession);
+    conf.set(TezConfiguration.TEZ_AM_STAGING_DIR, STAGING_DIR.getAbsolutePath());
     TezClientForTest client = new TezClientForTest("test", conf, lrs, null);
 
     ApplicationId appId1 = ApplicationId.newInstance(0, 1);
@@ -176,11 +195,11 @@ public class TestTezClient {
         .thenReturn(GetAMStatusResponseProto.newBuilder().setStatus(TezAppMasterStatusProto.RUNNING).build());
 
     client.sessionAmProxy = sessionAmProxy;
-    client.mockTezYarnClient = new TezYarnClient(yarnClient);
+    client.frameworkClient = new TezYarnClientForTest(yarnClient, client);
     client.mockYarnClient = yarnClient;
     client.mockAppId = appId1;
-    
-    return client;    
+
+    return client;
   }
   
   @Test (timeout = 5000)
@@ -885,7 +904,7 @@ public class TestTezClient {
     when(yarnClient.createApplication().getNewApplicationResponse().getApplicationId()).thenReturn(appId1);
     when(yarnClient.getApplicationReport(appId1)).thenReturn(mockReport);
     TezYarnClient tezClient = new TezYarnClient(yarnClient);
-    tezClient.init(new TezConfiguration(false), new YarnConfiguration());
+    tezClient.init(new TezConfiguration(false));
     try {
       tezClient.getApplicationReport(appId1);
       fail("getApplicationReport should have thrown");
@@ -984,7 +1003,6 @@ public class TestTezClient {
     String val = "hostname:2181";
     conf.set("yarn.resourcemanager.zk-address", val);
 
-    ConfigurationProto confProto = null;
     //Test that Exception is not thrown by createFinalConfProtoForApp
     TezClientUtils.createFinalConfProtoForApp(conf, null);
   }
