@@ -26,8 +26,8 @@ import static org.junit.Assert.assertTrue;
 import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.IOException;
+import java.net.MalformedURLException;
 import java.net.URL;
-import java.net.URLClassLoader;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -47,8 +47,10 @@ import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.permission.FsPermission;
 import org.apache.hadoop.hdfs.DistributedFileSystem;
 import org.apache.hadoop.io.DataInputByteBuffer;
+import org.apache.hadoop.io.Text;
 import org.apache.hadoop.security.Credentials;
 import org.apache.hadoop.security.token.Token;
+import org.apache.hadoop.security.token.TokenIdentifier;
 import org.apache.hadoop.yarn.api.ApplicationConstants;
 import org.apache.hadoop.yarn.api.ApplicationConstants.Environment;
 import org.apache.hadoop.yarn.api.records.ApplicationId;
@@ -81,6 +83,8 @@ import org.junit.Test;
 public class TestTezClientUtils {
   private static String TEST_ROOT_DIR = "target" + Path.SEPARATOR
       + TestTezClientUtils.class.getName() + "-tmpDir";
+  private static final File STAGING_DIR = new File(System.getProperty("test.build.data", "target"),
+      TestTezClientUtils.class.getName()).getAbsoluteFile();
   /**
    * 
    */
@@ -128,12 +132,29 @@ public class TestTezClientUtils {
     TezClientUtils.setupTezJarsLocalResources(conf, credentials, resources);
   }
 
-  /**
-   *
-   */
-  @Test (timeout=10000)
+  private static List<URL> getDirAndFileURL() throws MalformedURLException {
+    String[] classpaths = System.getProperty("java.class.path")
+        .split(System.getProperty("path.separator"));
+    List<URL> urls = new ArrayList<>(2);
+    File lastFile = null;
+    // Add one file and one directory.
+    for (String path : classpaths) {
+      URL url = new URL("file://" + path);
+      File file = FileUtils.toFile(url);
+      if (lastFile == null) {
+        lastFile = file;
+        urls.add(url);
+      } else if (lastFile.isDirectory() != file.isDirectory()) {
+        urls.add(url);
+        break;
+      }
+    }
+    return urls;
+  }
+
+  @Test (timeout=20000)
   public void validateSetTezJarLocalResourcesDefinedExistingDirectory() throws Exception {
-    URL[] cp = ((URLClassLoader)ClassLoader.getSystemClassLoader()).getURLs();
+    List<URL> cp = getDirAndFileURL();
     StringBuffer buffer = new StringBuffer();
     for (URL url : cp) {
       buffer.append(url.toExternalForm());
@@ -147,22 +168,27 @@ public class TestTezClientUtils {
         localizedMap);
     Assert.assertFalse(usingArchive);
     Set<String> resourceNames = localizedMap.keySet();
+    boolean assertedDir = false;
+    boolean assertedFile = false;
     for (URL url : cp) {
       File file = FileUtils.toFile(url);
-      if (file.isDirectory()){
+      if (file.isDirectory()) {
         String[] firList = file.list();
         for (String fileNme : firList) {
           File innerFile = new File(file, fileNme);
           if (!innerFile.isDirectory()){
             assertTrue(resourceNames.contains(innerFile.getName()));
+            assertedDir = true;
           }
           // not supporting deep hierarchies 
         }
-      }
-      else {
+      } else {
         assertTrue(resourceNames.contains(file.getName()));
+        assertedFile = true;
       }
     }
+    assertTrue(assertedDir);
+    assertTrue(assertedFile);
   }
 
   /**
@@ -171,7 +197,7 @@ public class TestTezClientUtils {
    */
   @Test (timeout=5000)
   public void validateSetTezJarLocalResourcesDefinedExistingDirectoryIgnored() throws Exception {
-    URL[] cp = ((URLClassLoader)ClassLoader.getSystemClassLoader()).getURLs();
+    List<URL> cp = getDirAndFileURL();
     StringBuffer buffer = new StringBuffer();
     for (URL url : cp) {
       buffer.append(url.toExternalForm());
@@ -190,9 +216,9 @@ public class TestTezClientUtils {
    * 
    * @throws Exception
    */
-  @Test (timeout=5000)
+  @Test (timeout=20000)
   public void validateSetTezJarLocalResourcesDefinedExistingDirectoryIgnoredSetToFalse() throws Exception {
-    URL[] cp = ((URLClassLoader)ClassLoader.getSystemClassLoader()).getURLs();
+    List<URL> cp = getDirAndFileURL();
     StringBuffer buffer = new StringBuffer();
     for (URL url : cp) {
       buffer.append(url.toExternalForm());
@@ -328,6 +354,7 @@ public class TestTezClientUtils {
   // ApplicationSubmissionContext
   public void testAppSubmissionContextForPriority() throws Exception {
     TezConfiguration tezConf = new TezConfiguration();
+    tezConf.set(TezConfiguration.TEZ_AM_STAGING_DIR, STAGING_DIR.getAbsolutePath());
     int testpriority = 999;
     ApplicationId appId = ApplicationId.newInstance(1000, 1);
     Credentials credentials = new Credentials();
@@ -378,6 +405,7 @@ public class TestTezClientUtils {
   public void testSessionTokenInAmClc() throws IOException, YarnException {
 
     TezConfiguration tezConf = new TezConfiguration();
+    tezConf.set(TezConfiguration.TEZ_AM_STAGING_DIR, STAGING_DIR.getAbsolutePath());
 
     ApplicationId appId = ApplicationId.newInstance(1000, 1);
     DAG dag = DAG.create("testdag");
@@ -415,6 +443,7 @@ public class TestTezClientUtils {
 
     TezConfiguration tezConf = new TezConfiguration();
     tezConf.set(TezConfiguration.TEZ_AM_LOG_LEVEL, "WARN");
+    tezConf.set(TezConfiguration.TEZ_AM_STAGING_DIR, STAGING_DIR.getAbsolutePath());
 
     ApplicationId appId = ApplicationId.newInstance(1000, 1);
     Credentials credentials = new Credentials();
@@ -455,6 +484,7 @@ public class TestTezClientUtils {
     TezConfiguration tezConf = new TezConfiguration();
     tezConf.set(TezConfiguration.TEZ_AM_LOG_LEVEL,
         "WARN;org.apache.hadoop.ipc=DEBUG;org.apache.hadoop.security=DEBUG");
+    tezConf.set(TezConfiguration.TEZ_AM_STAGING_DIR, STAGING_DIR.getAbsolutePath());
 
     ApplicationId appId = ApplicationId.newInstance(1000, 1);
     Credentials credentials = new Credentials();
@@ -890,5 +920,29 @@ public class TestTezClientUtils {
 
   }
 
+  @Test
+  public void testSessionCredentialsMergedBeforeAmConfigCredentials() throws Exception {
+    TezConfiguration conf = new TezConfiguration();
+    Text tokenType = new Text("TEST_TOKEN_TYPE");
+    Text tokenKind = new Text("TEST_TOKEN_KIND");
+    Text tokenService = new Text("TEST_TOKEN_SERVICE");
 
+    Credentials amConfigCredentials = new Credentials();
+    amConfigCredentials.addToken(tokenType,
+        new Token<>("id1".getBytes(), null, tokenKind, tokenService));
+
+    Credentials sessionCredentials = new Credentials();
+    Token<TokenIdentifier> sessionToken =
+        new Token<>("id2".getBytes(), null, tokenKind, tokenService);
+    sessionCredentials.addToken(tokenType, sessionToken);
+
+    AMConfiguration amConfig = new AMConfiguration(conf, null, amConfigCredentials);
+
+    Credentials amLaunchCredentials =
+        TezClientUtils.prepareAmLaunchCredentials(amConfig, sessionCredentials, conf, null);
+
+    // if there is another token in am conf creds of the same token type,
+    // session token should be applied while creating ContainerLaunchContext
+    Assert.assertEquals(sessionToken, amLaunchCredentials.getToken(tokenType));
+  }
 }
