@@ -18,16 +18,18 @@
 
 package org.apache.tez.client;
 
-import java.io.File;
 import java.io.IOException;
 import java.net.InetAddress;
 import java.nio.ByteBuffer;
 import java.util.List;
+import java.util.Map;
+import java.util.Set;
+import java.util.function.BiFunction;
 
-import org.apache.hadoop.yarn.api.ApplicationConstants;
 import org.apache.hadoop.yarn.api.protocolrecords.GetNewApplicationResponse;
 import org.apache.hadoop.yarn.api.records.ContainerId;
 import org.apache.hadoop.yarn.api.records.FinalApplicationStatus;
+import org.apache.hadoop.yarn.api.records.LocalResource;
 import org.apache.hadoop.yarn.client.api.YarnClientApplication;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.exceptions.YarnException;
@@ -45,21 +47,31 @@ import org.apache.hadoop.yarn.api.records.ApplicationAttemptId;
 import org.apache.hadoop.yarn.api.records.ApplicationReport;
 import org.apache.hadoop.yarn.api.records.YarnApplicationState;
 import org.apache.hadoop.yarn.util.SystemClock;
+import org.apache.tez.common.AsyncDispatcher;
 import org.apache.tez.common.TezCommonUtils;
 import org.apache.tez.common.TezUtilsInternal;
+import org.apache.tez.dag.api.DAGSubmissionTimedOut;
+import org.apache.tez.dag.api.DagTypeConverters;
 import org.apache.tez.dag.api.TezConfiguration;
 import org.apache.tez.dag.api.TezException;
+import org.apache.tez.dag.api.client.DAGClient;
 import org.apache.tez.dag.api.client.DAGClientHandler;
-import org.apache.tez.dag.api.records.DAGProtos;
+import org.apache.tez.dag.api.client.DAGClientImpl;
+import org.apache.tez.dag.api.client.DAGClientImplLocal;
+import org.apache.tez.dag.api.client.DAGStatus;
+import org.apache.tez.dag.api.client.StatusGetOpts;
+import org.apache.tez.dag.api.client.rpc.DAGClientAMProtocolRPC.SubmitDAGRequestProto;
 import org.apache.tez.dag.api.records.DAGProtos.AMPluginDescriptorProto;
 import org.apache.tez.dag.app.AppContext;
 import org.apache.tez.dag.app.DAGAppMaster;
 import org.apache.tez.dag.app.DAGAppMasterState;
+import org.apache.tez.dag.app.LocalDAGAppMaster;
 import org.apache.tez.dag.app.dag.DAG;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import com.google.common.annotations.VisibleForTesting;
+import com.google.protobuf.ServiceException;
 
 public class LocalClient extends FrameworkClient {
   public static final Logger LOG = LoggerFactory.getLogger(LocalClient.class);
@@ -74,6 +86,8 @@ public class LocalClient extends FrameworkClient {
   private boolean isSession;
   private TezApiVersionInfo versionInfo = new TezApiVersionInfo();
   private volatile Throwable amFailException = null;
+  private boolean isLocalWithoutNetwork;
+
   private static final String localModeDAGSchedulerClassName =
       "org.apache.tez.dag.app.dag.impl.DAGSchedulerNaturalOrderControlled";
 
@@ -81,9 +95,8 @@ public class LocalClient extends FrameworkClient {
   }
 
   @Override
-  public void init(TezConfiguration tezConf, YarnConfiguration yarnConf) {
+  public void init(TezConfiguration tezConf) {
     this.conf = tezConf;
-    tezConf.set("fs.defaultFS", "file:///");
     // Tez libs already in the client's classpath
     this.conf.setBoolean(TezConfiguration.TEZ_IGNORE_LIB_URIS, true);
     this.conf.set(TezConfiguration.TEZ_AM_DAG_SCHEDULER_CLASS, localModeDAGSchedulerClassName);
@@ -92,6 +105,10 @@ public class LocalClient extends FrameworkClient {
 
     // disable web service for local mode.
     this.conf.setBoolean(TezConfiguration.TEZ_AM_WEBSERVICE_ENABLE, false);
+
+    this.isLocalWithoutNetwork =
+        tezConf.getBoolean(TezConfiguration.TEZ_LOCAL_MODE_WITHOUT_NETWORK,
+            TezConfiguration.TEZ_LOCAL_MODE_WITHOUT_NETWORK_DEFAULT);
   }
 
 
@@ -140,7 +157,9 @@ public class LocalClient extends FrameworkClient {
   @Override
   public void killApplication(ApplicationId appId) {
     try {
-      clientHandler.shutdownAM();
+      if (clientHandler != null){
+        clientHandler.shutdownAM();
+      }
     } catch (TezException e) {
       throw new RuntimeException(e);
     }
@@ -173,7 +192,6 @@ public class LocalClient extends FrameworkClient {
     report.setYarnApplicationState(convertDAGAppMasterState(dagAppMaster.getState()));
     report.setFinalApplicationStatus(convertDAGAppMasterStateToFinalYARNState(dagAppMaster.getState()));
 
-
     List<String> diagnostics = dagAppMaster.getDiagnostics();
     if (diagnostics != null) {
       report.setDiagnostics(diagnostics.toString());
@@ -286,19 +304,34 @@ public class LocalClient extends FrameworkClient {
         try {
           ApplicationId appId = appContext.getApplicationId();
 
-          // Set up working directory for DAGAppMaster
+          // Set up working directory for DAGAppMaster.
+          // The staging directory may be on the default file system, which may or may not
+          // be the local FS. For example, when using testing Hive against a pseudo-distributed
+          // cluster, it's useful for the default FS to be HDFS. Hive then puts its scratch
+          // directories on HDFS, and sets the Tez staging directory to be the session's
+          // scratch directory.
+          //
+          // To handle this case, we need to copy over the staging data back onto the
+          // local file system, where the rest of the Tez Child code expects it.
+          //
+          // NOTE: we base the local working directory path off of the staging path, even
+          // though it might be on a different file system. Typically they're both in a
+          // path starting with /tmp, but in the future we may want to use a different
+          // temp directory locally.
           Path staging = TezCommonUtils.getTezSystemStagingPath(conf, appId.toString());
-          Path userDir = TezCommonUtils.getTezSystemStagingPath(conf, appId.toString()+"_wd");
+          FileSystem stagingFs = staging.getFileSystem(conf);
+
+          FileSystem localFs = FileSystem.getLocal(conf);
+          Path userDir = localFs.makeQualified(new Path(staging.toUri().getPath() + "_wd"));
           LOG.info("Using working directory: " + userDir.toUri().getPath());
 
-          FileSystem fs = FileSystem.get(conf);
           // copy data from staging directory to working directory to simulate the resource localizing
-          FileUtil.copy(fs, staging, fs, userDir, false, conf);
+          FileUtil.copy(stagingFs, staging, localFs, userDir, false, conf);
           // Prepare Environment
           Path logDir = new Path(userDir, "localmode-log-dir");
           Path localDir = new Path(userDir, "localmode-local-dir");
-          fs.mkdirs(logDir);
-          fs.mkdirs(localDir);
+          localFs.mkdirs(logDir);
+          localFs.mkdirs(localDir);
 
           UserGroupInformation.setConfiguration(conf);
           // Add session specific credentials to the AM credentials.
@@ -321,12 +354,12 @@ public class LocalClient extends FrameworkClient {
 
           dagAppMaster =
               createDAGAppMaster(applicationAttemptId, cId, currentHost, nmPort, nmHttpPort,
-                  new SystemClock(), appSubmitTime, isSession, userDir.toUri().getPath(),
+                  SystemClock.getInstance(), appSubmitTime, isSession, userDir.toUri().getPath(),
                   new String[] {localDir.toUri().getPath()}, new String[] {logDir.toUri().getPath()},
                   amCredentials, UserGroupInformation.getCurrentUser().getShortUserName());
           DAGAppMaster.initAndStartAppMaster(dagAppMaster, conf);
           clientHandler = new DAGClientHandler(dagAppMaster);
-
+          ((AsyncDispatcher)dagAppMaster.getDispatcher()).setDrainEventsOnStop();
         } catch (Throwable t) {
           LOG.error("Error starting DAGAppMaster", t);
           if (dagAppMaster != null) {
@@ -357,30 +390,67 @@ public class LocalClient extends FrameworkClient {
 
     // Read in additional information about external services
     AMPluginDescriptorProto amPluginDescriptorProto =
-        getPluginDescriptorInfo(conf, applicationAttemptId.getApplicationId().toString());
+        TezUtilsInternal.readUserSpecifiedTezConfiguration(userDir)
+            .getAmPluginDescriptor();
 
-
-    return new DAGAppMaster(applicationAttemptId, cId, currentHost, nmPort, nmHttpPort,
-        new SystemClock(), appSubmitTime, isSession, userDir, localDirs, logDirs,
-        versionInfo.getVersion(), credentials, jobUserName, amPluginDescriptorProto);
+    return isLocalWithoutNetwork
+      ? new LocalDAGAppMaster(applicationAttemptId, cId, currentHost, nmPort, nmHttpPort,
+          SystemClock.getInstance(), appSubmitTime, isSession, userDir, localDirs, logDirs,
+          versionInfo.getVersion(), credentials, jobUserName, amPluginDescriptorProto)
+      : new DAGAppMaster(applicationAttemptId, cId, currentHost, nmPort, nmHttpPort,
+          SystemClock.getInstance(), appSubmitTime, isSession, userDir, localDirs, logDirs,
+          versionInfo.getVersion(), credentials, jobUserName, amPluginDescriptorProto);
   }
 
-  private AMPluginDescriptorProto getPluginDescriptorInfo(Configuration conf,
-                                                          String applicationIdString) throws
-      IOException {
-    Path tezSysStagingPath = TezCommonUtils
-        .getTezSystemStagingPath(conf, applicationIdString);
-    // Remove the filesystem qualifier.
-    String unqualifiedPath = tezSysStagingPath.toUri().getPath();
-
-    DAGProtos.ConfigurationProto confProto =
-        TezUtilsInternal
-            .readUserSpecifiedTezConfiguration(unqualifiedPath);
-    AMPluginDescriptorProto amPluginDescriptorProto = null;
-    if (confProto.hasAmPluginDescriptor()) {
-      amPluginDescriptorProto = confProto.getAmPluginDescriptor();
+  @Override
+  public TezAppMasterStatus getAMStatus(Configuration configuration, ApplicationId appId,
+      UserGroupInformation ugi) throws TezException, ServiceException, IOException {
+    if (isLocalWithoutNetwork) {
+      if (clientHandler == null) {
+        return TezAppMasterStatus.INITIALIZING;
+      }
+      return clientHandler.getTezAppMasterStatus();
     }
-    return amPluginDescriptorProto;
+    return super.getAMStatus(configuration, appId, ugi);
   }
 
+  @Override
+  public DAGClient submitDag(org.apache.tez.dag.api.DAG dag, SubmitDAGRequestProto request,
+      String clientName, ApplicationId sessionAppId, long clientTimeout, UserGroupInformation ugi,
+      TezConfiguration tezConf) throws IOException, TezException, DAGSubmissionTimedOut {
+
+    Map<String, LocalResource> additionalResources = null;
+    if (request.hasAdditionalAmResources()) {
+      additionalResources =
+          DagTypeConverters.convertFromPlanLocalResources(request.getAdditionalAmResources());
+    }
+
+    String dagId = dagAppMaster.submitDAGToAppMaster(request.getDAGPlan(), additionalResources);
+
+    return isLocalWithoutNetwork
+      ? new DAGClientImplLocal(sessionAppId, dagId, tezConf, this,
+          ugi, new BiFunction<Set<StatusGetOpts>, Long, DAGStatus>() {
+            @Override
+            public DAGStatus apply(Set<StatusGetOpts> statusOpts, Long timeout) {
+              try {
+                return clientHandler.getDAGStatus(dagId, statusOpts, timeout);
+              } catch (TezException e) {
+                throw new RuntimeException(e);
+              }
+            }
+          })
+      : new DAGClientImpl(sessionAppId, dagId, tezConf, this, ugi);
+  }
+
+  @Override
+  public boolean shutdownSession(Configuration configuration, ApplicationId sessionAppId,
+      UserGroupInformation ugi) throws TezException, IOException, ServiceException {
+    if (isLocalWithoutNetwork) {
+      if (clientHandler != null){
+        clientHandler.shutdownAM();
+      }
+      return true;
+    }
+    return super.shutdownSession(configuration, sessionAppId, ugi);
+  }
 }
