@@ -42,6 +42,9 @@ import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import org.apache.commons.lang.StringUtils;
 import org.apache.commons.lang.exception.ExceptionUtils;
+import org.apache.tez.Utils;
+import org.apache.tez.common.GuavaShim;
+import org.apache.tez.common.ProgressHelper;
 import org.apache.tez.common.TezUtilsInternal;
 import org.apache.tez.common.counters.LimitExceededException;
 import org.apache.tez.dag.app.dag.event.DAGEventTerminateDag;
@@ -54,6 +57,7 @@ import org.apache.hadoop.classification.InterfaceAudience.Private;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.security.Credentials;
 import org.apache.hadoop.security.UserGroupInformation;
+import org.apache.hadoop.yarn.api.ApplicationConstants;
 import org.apache.hadoop.yarn.api.records.LocalResource;
 import org.apache.hadoop.yarn.event.EventHandler;
 import org.apache.hadoop.yarn.state.InvalidStateTransitonException;
@@ -64,6 +68,8 @@ import org.apache.hadoop.yarn.state.StateMachineFactory;
 import org.apache.hadoop.yarn.util.Clock;
 import org.apache.tez.common.ATSConstants;
 import org.apache.tez.common.ReflectionUtils;
+import org.apache.tez.common.TezCommonUtils;
+import org.apache.tez.common.counters.AggregateTezCounters;
 import org.apache.tez.common.counters.DAGCounter;
 import org.apache.tez.common.counters.TezCounters;
 import org.apache.tez.dag.api.DagTypeConverters;
@@ -134,7 +140,7 @@ import org.apache.tez.dag.utils.TezBuilderUtils;
 import org.apache.tez.runtime.api.OutputCommitter;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
@@ -160,8 +166,9 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
   // TODO Recovery
   //private final List<AMInfo> amInfos;
   private final Lock dagStatusLock = new ReentrantLock();
-  private final Condition dagCompletionCondition = dagStatusLock.newCondition();
+  private final Condition dagStateChangedCondition = dagStatusLock.newCondition();
   private final AtomicBoolean isFinalState = new AtomicBoolean(false);
+  private final AtomicBoolean runningStatusYetToBeConsumed = new AtomicBoolean(false);
   private final Lock readLock;
   private final Lock writeLock;
   private final String dagName;
@@ -220,6 +227,7 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
   private TaskSpecificLaunchCmdOption taskSpecificLaunchCmdOption;
 
   private static final DagStateChangedCallback STATE_CHANGED_CALLBACK = new DagStateChangedCallback();
+  private String[] logDirs;
 
   @VisibleForTesting
   Map<OutputKey, ListenableFuture<Void>> commitFutures
@@ -565,6 +573,8 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
 
   private void augmentStateMachine() {
     stateMachine
+        .registerStateEnteredCallback(DAGState.RUNNING,
+            STATE_CHANGED_CALLBACK)
         .registerStateEnteredCallback(DAGState.SUCCEEDED,
             STATE_CHANGED_CALLBACK)
         .registerStateEnteredCallback(DAGState.FAILED,
@@ -579,10 +589,22 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
       implements OnStateChangedCallback<DAGState, DAGImpl> {
     @Override
     public void onStateChanged(DAGImpl dag, DAGState dagState) {
-      dag.isFinalState.set(true);
+      switch(dagState) {
+      case RUNNING:
+        dag.runningStatusYetToBeConsumed.set(true);
+        break;
+      case SUCCEEDED:
+      case FAILED:
+      case KILLED:
+      case ERROR:
+        dag.isFinalState.set(true);
+        break;
+      default:
+        break;
+      }
       dag.dagStatusLock.lock();
       try {
-        dag.dagCompletionCondition.signal();
+        dag.dagStateChangedCondition.signal();
       } finally {
         dag.dagStatusLock.unlock();
       }
@@ -700,7 +722,7 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
       updateCpuCounters();
       TezCounters counters = new TezCounters();
       counters.incrAllCounters(dagCounters);
-      return incrTaskCounters(counters, vertices.values());
+      return aggrTaskCounters(counters, vertices.values());
 
     } finally {
       readLock.unlock();
@@ -732,7 +754,7 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
       updateCpuCounters();
       TezCounters counters = new TezCounters();
       counters.incrAllCounters(dagCounters);
-      return incrTaskCounters(counters, vertices.values());
+      return aggrTaskCounters(counters, vertices.values());
 
     } finally {
       readLock.unlock();
@@ -748,10 +770,10 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
     return false;
   }
 
-  public static TezCounters incrTaskCounters(
+  public static TezCounters aggrTaskCounters(
       TezCounters counters, Collection<Vertex> vertices) {
     for (Vertex vertex : vertices) {
-      counters.incrAllCounters(vertex.getAllCounters());
+      counters.aggrAllCounters(vertex.getAllCounters());
     }
     return counters;
   }
@@ -802,19 +824,30 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
   public float getProgress() {
     this.readLock.lock();
     try {
-      float progress = 0.0f;
+      float accProg = 0.0f;
+      float dagProgress = 0.0f;
+      int verticesCount = getVertices().size();
       for (Vertex v : getVertices().values()) {
         float vertexProgress = v.getProgress();
-        if (vertexProgress >= 0.0f && vertexProgress <= 1.0f) {
-          progress += vertexProgress;
+        if (LOG.isDebugEnabled()) {
+          if (!ProgressHelper.isProgressWithinRange(vertexProgress)) {
+            LOG.debug("progress update: Vertex progress is invalid range"
+                + "; v={}, progress={}", v.getName(), vertexProgress);
+          }
+        }
+        accProg += ProgressHelper.processProgress(vertexProgress);
+      }
+      if (LOG.isDebugEnabled()) {
+        if (verticesCount == 0) {
+          LOG.debug("progress update: DAGImpl getProgress() returns 0.0f: "
+              + "vertices count is 0");
         }
       }
-      float dagProgress = progress / getTotalVertices();
-      if (dagProgress >= 0.0f && progress <= 1.0f) {
-        return dagProgress;
-      } else {
-        return 0.0f;
+      if (verticesCount > 0) {
+        dagProgress =
+            ProgressHelper.processProgress(accProg / verticesCount);
       }
+      return dagProgress;
     } finally {
       this.readLock.unlock();
     }
@@ -931,7 +964,11 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
         if (isFinalState.get()) {
           break;
         }
-        nanosLeft = dagCompletionCondition.awaitNanos(timeoutNanos);
+        if (runningStatusYetToBeConsumed.compareAndSet(true, false)) {
+          // No need to wait further, as state just got changed to RUNNING
+          break;
+        }
+        nanosLeft = dagStateChangedCondition.awaitNanos(timeoutNanos);
       } catch (InterruptedException e) {
         throw new TezException("Interrupted while waiting for dag to complete", e);
       } finally {
@@ -1128,7 +1165,7 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
       }
       for (Map.Entry<OutputKey,CallableEvent> entry : commitEvents.entrySet()) {
         ListenableFuture<Void> commitFuture = appContext.getExecService().submit(entry.getValue());
-        Futures.addCallback(commitFuture, entry.getValue().getCallback());
+        Futures.addCallback(commitFuture, entry.getValue().getCallback(), GuavaShim.directExecutor());
         commitFutures.put(entry.getKey(), commitFuture);
       }
     }
@@ -1407,7 +1444,7 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
       updateCpuCounters();
       TezCounters counters = null;
       try {
-        counters = getAllCounters();
+        counters = constructFinalFullcounters();
       } catch (LimitExceededException e) {
         addDiagnostic("Counters limit exceeded: " + e.getMessage());
         finalState = DAGState.FAILED;
@@ -1617,16 +1654,19 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
       if (!groupInfo.outputs.isEmpty()) {
         // shared outputs
         for (String vertexName : groupInfo.groupMembers) {
-          if (LOG.isDebugEnabled()) {
-            LOG.debug("Setting shared outputs for group: " + groupName +
-                " on vertex: " + vertexName);
-          }
+          LOG.debug("Setting shared outputs for group: {} on vertex: {}", groupName, vertexName);
           Vertex v = getVertex(vertexName);
           v.addSharedOutputs(groupInfo.outputs);
         }
       }
     }
 
+    // This is going to override the previously generated file
+    // which didn't have the priorities
+    if (getConf().getBoolean(TezConfiguration.TEZ_GENERATE_DEBUG_ARTIFACTS,
+        TezConfiguration.TEZ_GENERATE_DEBUG_ARTIFACTS_DEFAULT)) {
+      Utils.generateDAGVizFile(this, jobPlan, logDirs, dagScheduler);
+    }
     return DAGState.INITED;
   }
 
@@ -1876,17 +1916,18 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
         // Already constructed. Just return.
         return;
       }
-      this.constructFinalFullcounters();
+      this.fullCounters = this.constructFinalFullcounters();
     }
   }
 
   @Private
-  public void constructFinalFullcounters() {
-    this.fullCounters = new TezCounters();
-    this.fullCounters.incrAllCounters(dagCounters);
+  public TezCounters constructFinalFullcounters() {
+    final AggregateTezCounters aggregateTezCounters = new AggregateTezCounters();
+    aggregateTezCounters.aggrAllCounters(dagCounters);
     for (Vertex v : this.vertices.values()) {
-      this.fullCounters.incrAllCounters(v.getAllCounters());
+      aggregateTezCounters.aggrAllCounters(v.getAllCounters());
     }
+    return aggregateTezCounters;
   }
 
   /**
@@ -2138,7 +2179,8 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
                 };
               };
               ListenableFuture<Void> groupCommitFuture = appContext.getExecService().submit(groupCommitCallableEvent);
-              Futures.addCallback(groupCommitFuture, groupCommitCallableEvent.getCallback());
+              Futures.addCallback(groupCommitFuture, groupCommitCallableEvent.getCallback(),
+                  GuavaShim.directExecutor());
               commitFutures.put(outputKey, groupCommitFuture);
             }
           }
@@ -2388,6 +2430,11 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
     }
   }
 
+  @Override
+  public DAGScheduler getDAGScheduler() {
+    return dagScheduler;
+  }
+
   // output of either vertex or vertex group
   public static class OutputKey {
     String outputName;
@@ -2466,4 +2513,17 @@ public class DAGImpl implements org.apache.tez.dag.app.dag.DAG,
       eventHandler.handle(new DAGEventCommitCompleted(dagId, outputKey, false, t));
     }
   }
+
+  public String[] getLogDirs() {
+    if (logDirs == null) {
+      logDirs = TezCommonUtils
+          .getTrimmedStrings(System.getenv(ApplicationConstants.Environment.LOG_DIRS.name()));
+    }
+    return logDirs;
+  }
+
+  public DAGImpl setLogDirs(String[] logDirs) {
+    this.logDirs = logDirs;
+    return this;
+  }
 }
