@@ -46,11 +46,10 @@ public class DAGSchedulerNaturalOrder extends DAGScheduler {
   public void scheduleTaskEx(DAGEventSchedulerUpdate event) {
     TaskAttempt attempt = event.getAttempt();
     Vertex vertex = dag.getVertex(attempt.getVertexID());
-    int vertexDistanceFromRoot = vertex.getDistanceFromRoot();
 
     // natural priority. Handles failures and retries.
-    int priorityLowLimit = ((vertexDistanceFromRoot + 1) * dag.getTotalVertices() * 3) + (vertex.getVertexId().getId() * 3);
-    int priorityHighLimit = priorityLowLimit - 2;
+    int priorityLowLimit = getPriorityLowLimit(dag, vertex);
+    int priorityHighLimit = getPriorityHighLimit(dag, vertex);
 
     if (LOG.isDebugEnabled()) {
       LOG.debug("Scheduling " + attempt.getID() + " between priorityLow: " + priorityLowLimit
