@@ -32,7 +32,7 @@ import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReadWriteLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.common.base.Predicate;
 import com.google.common.collect.Maps;
 
@@ -126,6 +126,7 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
   private final TezTaskID taskId;
   private Map<TezTaskAttemptID, TaskAttempt> attempts;
   protected final int maxFailedAttempts;
+  protected final int maxAttempts; //overall max number of attempts (consider preempted task attempts)
   protected final Clock clock;
   private final Vertex vertex;
   private final Lock readLock;
@@ -152,10 +153,14 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
   // track the status of TaskAttempt (true mean completed, false mean uncompleted)
   private final Map<Integer, Boolean> taskAttemptStatus = new HashMap<Integer,Boolean>();
 
+  // The set of nodes with active running attempts at the time of the latest attempt for
+  // this task was scheduled. This set is empty when scheduling original task attempt, and
+  // non-empty scheduling a speculative attempt, in which case scheduler should avoid
+  // scheduling the speculative attempt onto node(s) recorded in this set.
   private final Set<NodeId> nodesWithRunningAttempts = Collections
       .newSetFromMap(new ConcurrentHashMap<NodeId, Boolean>());
 
-  private static final SingleArcTransition<TaskImpl , TaskEvent>
+  private static final MultipleArcTransition<TaskImpl, TaskEvent, TaskStateInternal>
      ATTEMPT_KILLED_TRANSITION = new AttemptKilledTransition();
   private static final SingleArcTransition<TaskImpl, TaskEvent>
      KILL_TRANSITION = new KillTransition();
@@ -189,7 +194,7 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
      .addTransition(TaskStateInternal.SCHEDULED, TaskStateInternal.KILL_WAIT,
          TaskEventType.T_TERMINATE,
          KILL_TRANSITION)
-     .addTransition(TaskStateInternal.SCHEDULED, TaskStateInternal.SCHEDULED,
+     .addTransition(TaskStateInternal.SCHEDULED, EnumSet.of(TaskStateInternal.SCHEDULED, TaskStateInternal.FAILED),
          TaskEventType.T_ATTEMPT_KILLED, ATTEMPT_KILLED_TRANSITION)
      .addTransition(TaskStateInternal.SCHEDULED,
         EnumSet.of(TaskStateInternal.SCHEDULED, TaskStateInternal.FAILED),
@@ -212,7 +217,7 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
         EnumSet.of(TaskStateInternal.SUCCEEDED),
         TaskEventType.T_ATTEMPT_SUCCEEDED,
         new AttemptSucceededTransition())
-    .addTransition(TaskStateInternal.RUNNING, TaskStateInternal.RUNNING,
+    .addTransition(TaskStateInternal.RUNNING, EnumSet.of(TaskStateInternal.RUNNING, TaskStateInternal.FAILED),
         TaskEventType.T_ATTEMPT_KILLED,
         ATTEMPT_KILLED_TRANSITION)
     .addTransition(TaskStateInternal.RUNNING,
@@ -376,6 +381,7 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
     writeLock = readWriteLock.writeLock();
     this.attempts = Collections.emptyMap();
     maxFailedAttempts = vertex.getVertexConfig().getMaxFailedTaskAttempts();
+    maxAttempts = vertex.getVertexConfig().getMaxTaskAttempts();
     taskId = TezTaskID.getInstance(vertexId, taskIndex);
     this.taskCommunicatorManagerInterface = taskCommunicatorManagerInterface;
     this.taskHeartbeatHandler = thh;
@@ -584,6 +590,11 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
     }
   }
 
+  @Override
+  public Set<NodeId> getNodesWithRunningAttempts() {
+    return nodesWithRunningAttempts;
+  }
+
   @VisibleForTesting
   public TaskStateInternal getInternalState() {
     readLock.lock();
@@ -679,9 +690,7 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
   public boolean canCommit(TezTaskAttemptID taskAttemptID) {
     writeLock.lock();
     try {
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Commit go/no-go request from " + taskAttemptID);
-      }
+      LOG.debug("Commit go/no-go request from {}", taskAttemptID);
       TaskState state = getState();
       if (state == TaskState.SCHEDULED) {
         // the actual running task ran and is done and asking for commit. we are still stuck 
@@ -719,9 +728,7 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
         }
       } else {
         if (commitAttempt.equals(taskAttemptID)) {
-          if (LOG.isDebugEnabled()) {
-            LOG.debug(taskAttemptID + " already given a go for committing the task output.");
-          }
+          LOG.debug("{} already given a go for committing the task output.", taskAttemptID);
           return true;
         }
         // Don't think this can be a pluggable decision, so simply raise an
@@ -729,9 +736,7 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
         // Wait for commit attempt to succeed. Dont kill this. If commit
         // attempt fails then choose a different committer. When commit attempt
         // succeeds then this and others will be killed
-        if (LOG.isDebugEnabled()) {
-          LOG.debug(commitAttempt + " is current committer. Commit waiting for:  " + taskAttemptID);
-        }
+        LOG.debug("{} is current committer. Commit waiting for: {}", commitAttempt, taskAttemptID);
         return false;
       }
 
@@ -749,8 +754,8 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
         baseTaskSpec.getTaskConf());
     return new TaskAttemptImpl(attemptId, eventHandler,
         taskCommunicatorManagerInterface, conf, clock, taskHeartbeatHandler, appContext,
-        (failedAttempts > 0), taskResource, containerContext, leafVertex, getVertex(),
-        locationHint, taskSpec, schedulingCausalTA, nodesWithRunningAttempts);
+        (failedAttempts > 0), taskResource, containerContext, leafVertex, this,
+        locationHint, taskSpec, schedulingCausalTA);
   }
 
   @Override
@@ -767,7 +772,7 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
   }
 
   // This is always called in the Write Lock
-  private void addAndScheduleAttempt(TezTaskAttemptID schedulingCausalTA) {
+  private boolean addAndScheduleAttempt(TezTaskAttemptID schedulingCausalTA) {
     TaskAttempt attempt = createAttempt(attempts.size(), schedulingCausalTA);
     if (LOG.isDebugEnabled()) {
       LOG.debug("Created attempt " + attempt.getID());
@@ -785,13 +790,23 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
         Preconditions.checkArgument(attempts.put(attempt.getID(), attempt) == null,
             attempt.getID() + " already existed");
         break;
-
       default:
         Preconditions.checkArgument(attempts.put(attempt.getID(), attempt) == null,
             attempt.getID() + " already existed");
         break;
     }
 
+    if (maxAttempts > 0 && attempts.size() == maxAttempts) {
+      TaskImpl task = (TaskImpl) attempt.getTask();
+      LOG.error("Cannot schedule new attempt for task as max number of attempts ({}) reached: {}",
+          maxAttempts, task);
+
+      task.logJobHistoryTaskFailedEvent(TaskState.FAILED);
+      task.eventHandler.handle(new VertexEventTaskCompleted(task.taskId, TaskState.FAILED));
+
+      return false;
+    }
+
     // TODO: Recovery
     /*
     // Update nextATtemptNumber
@@ -810,7 +825,7 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
     // picture in mind
     eventHandler.handle(new DAGEventSchedulerUpdate(
         DAGEventSchedulerUpdate.UpdateType.TA_SCHEDULE, attempt));
-
+    return true;
   }
 
   @Override
@@ -998,7 +1013,9 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
       task.locationHint = scheduleEvent.getTaskLocationHint();
       task.baseTaskSpec = scheduleEvent.getBaseTaskSpec();
       // For now, initial scheduling dependency is due to vertex manager scheduling
-      task.addAndScheduleAttempt(null);
+      if (!task.addAndScheduleAttempt(null)) {
+        return TaskStateInternal.FAILED;
+      }
       return TaskStateInternal.SCHEDULED;
     }
   }
@@ -1020,9 +1037,27 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
         // find the oldest running attempt
         if (!ta.isFinished()) {
           earliestUnfinishedAttempt = ta;
-          task.nodesWithRunningAttempts.add(ta.getNodeId());
+          if (ta.getNodeId() != null) {
+            task.nodesWithRunningAttempts.add(ta.getNodeId());
+          }
+        } else {
+          if (TaskAttemptState.SUCCEEDED.equals(ta.getState())) {
+            LOG.info("Ignore speculation scheduling for task {} since it has succeeded with attempt {}.",
+                task.getTaskId(), ta.getID());
+            return;
+          }
         }
       }
+      if (earliestUnfinishedAttempt == null) {
+        // no running (or SUCCEEDED) task attempt at this moment, no need to schedule speculative attempt either
+        LOG.info("Ignore speculation scheduling since there is no running attempt on task {}.", task.getTaskId());
+        return;
+      }
+      if (task.commitAttempt != null) {
+        LOG.info("Ignore speculation scheduling for task {} since commit has started with commitAttempt {}.",
+            task.getTaskId(), task.commitAttempt);
+        return;
+      }
       task.addAndScheduleAttempt(earliestUnfinishedAttempt.getID());
     }
   }
@@ -1081,7 +1116,9 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
           LOG.info("Can not recover the successful task attempt, schedule new task attempt,"
               + "taskId=" + task.getTaskId());
           task.successfulAttempt = null;
-          task.addAndScheduleAttempt(successTaId);
+          if (!task.addAndScheduleAttempt(successTaId)) {
+            task.finished(TaskStateInternal.FAILED);
+          }
           task.eventHandler.handle(new TaskAttemptEventAttemptKilled(successTaId,
               errorMsg, TaskAttemptTerminationCause.TERMINATED_AT_RECOVERY, true));
           return TaskStateInternal.RUNNING;
@@ -1140,9 +1177,11 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
   }
 
   private static class AttemptKilledTransition implements
-      SingleArcTransition<TaskImpl, TaskEvent> {
+      MultipleArcTransition<TaskImpl, TaskEvent, TaskStateInternal> {
     @Override
-    public void transition(TaskImpl task, TaskEvent event) {
+    public TaskStateInternal transition(TaskImpl task, TaskEvent event) {
+      TaskStateInternal originalState = task.getInternalState();
+
       TaskEventTAUpdate castEvent = (TaskEventTAUpdate) event;
       task.addDiagnosticInfo("TaskAttempt " + castEvent.getTaskAttemptID().getId() + " killed");
       if (task.commitAttempt !=null &&
@@ -1170,8 +1209,11 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
         task.getVertex().incrementKilledTaskAttemptCount();
       }
       if (task.shouldScheduleNewAttempt()) {
-        task.addAndScheduleAttempt(castEvent.getTaskAttemptID());
+        if (!task.addAndScheduleAttempt(castEvent.getTaskAttemptID())) {
+          return task.finished(TaskStateInternal.FAILED);
+        }
       }
+      return originalState;
     }
   }
 
@@ -1231,8 +1273,10 @@ public class TaskImpl implements Task, EventHandler<TaskEvent> {
         if (task.shouldScheduleNewAttempt()) {
           LOG.info("Scheduling new attempt for task: " + task.getTaskId()
               + ", currentFailedAttempts: " + task.failedAttempts + ", maxFailedAttempts: "
-              + task.maxFailedAttempts);
-          task.addAndScheduleAttempt(getSchedulingCausalTA());
+              + task.maxFailedAttempts + ", maxAttempts: " + task.maxAttempts);
+          if (!task.addAndScheduleAttempt(getSchedulingCausalTA())){
+            return task.finished(TaskStateInternal.FAILED);
+          }
         }
       } else {
         if (castEvent.getTaskFailureType() == TaskFailureType.NON_FATAL) {
