@@ -19,10 +19,9 @@
 package org.apache.tez.dag.app.rm;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.common.util.concurrent.ThreadFactoryBuilder;
 import org.apache.commons.lang.mutable.MutableInt;
-import org.apache.commons.math3.random.RandomDataGenerator;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.hadoop.util.Time;
@@ -80,6 +79,7 @@ import java.util.concurrent.Future;
 import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.ScheduledThreadPoolExecutor;
+import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.TimeUnit;
 
 /**
@@ -95,7 +95,6 @@ public class DagAwareYarnTaskScheduler extends TaskScheduler
   private static final Logger LOG = LoggerFactory.getLogger(DagAwareYarnTaskScheduler.class);
   private static final Comparator<HeldContainer> PREEMPT_ORDER_COMPARATOR = new PreemptOrderComparator();
 
-  private final RandomDataGenerator random = new RandomDataGenerator();
   private AMRMClientAsyncWrapper client;
   private ScheduledExecutorService reuseExecutor;
   private ResourceCalculator resourceCalculator;
@@ -743,7 +742,7 @@ public class DagAwareYarnTaskScheduler extends TaskScheduler
 
       ++numHeartbeats;
       if (LOG.isDebugEnabled() || numHeartbeats % 50 == 1) {
-        LOG.info(constructPeriodicLog(freeResources));
+        LOG.debug(constructPeriodicLog(freeResources));
       }
 
       preemptedContainers = maybePreempt(freeResources);
@@ -1132,7 +1131,7 @@ public class DagAwareYarnTaskScheduler extends TaskScheduler
   private boolean maybeChangeNode(TaskRequest request, NodeId nodeId) {
     Object task = request.getTask();
     if (task instanceof TaskAttempt) {
-      Set<NodeId> nodesWithSiblingRunningAttempts = ((TaskAttempt) task).getNodesWithSiblingRunningAttempts();
+      Set<NodeId> nodesWithSiblingRunningAttempts = ((TaskAttempt) task).getTask().getNodesWithRunningAttempts();
       if (nodesWithSiblingRunningAttempts != null
           && nodesWithSiblingRunningAttempts.contains(nodeId)) {
         return true;
@@ -1543,7 +1542,8 @@ public class DagAwareYarnTaskScheduler extends TaskScheduler
     long getIdleExpirationTimestamp(long now) {
       if (idleExpirationTimestamp == 0) {
         if (idleContainerTimeoutMin > 0) {
-          idleExpirationTimestamp = now + random.nextLong(idleContainerTimeoutMin, idleContainerTimeoutMax);
+          idleExpirationTimestamp = now + (idleContainerTimeoutMin == idleContainerTimeoutMax ? idleContainerTimeoutMin
+            : ThreadLocalRandom.current().nextLong(idleContainerTimeoutMin, idleContainerTimeoutMax));
         } else {
           idleExpirationTimestamp = Long.MAX_VALUE;
         }
