@@ -28,6 +28,7 @@ import java.util.concurrent.Executors;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.Objects;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.util.concurrent.ThreadFactoryBuilder;
@@ -92,7 +93,7 @@ import org.apache.tez.dag.records.TaskAttemptTerminationCause;
 import org.apache.tez.hadoop.shim.HadoopShim;
 import org.apache.tez.hadoop.shim.HadoopShimsLoader;
 
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 
 
 public class TaskSchedulerManager extends AbstractService implements
@@ -264,9 +265,7 @@ public class TaskSchedulerManager extends AbstractService implements
   }
 
   public synchronized void handleEvent(AMSchedulerEvent sEvent) {
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("Processing the event " + sEvent.toString());
-    }
+    LOG.debug("Processing the event {}", sEvent);
     switch (sEvent.getType()) {
     case S_TA_LAUNCH_REQUEST:
       handleTaLaunchRequest((AMSchedulerEventTALaunchRequest) sEvent);
@@ -483,7 +482,7 @@ public class TaskSchedulerManager extends AbstractService implements
       TaskBasedLocationAffinity taskAffinity = locationHint.getAffinitizedTask();
       if (taskAffinity != null) {
         Vertex vertex = appContext.getCurrentDAG().getVertex(taskAffinity.getVertexName());
-        Preconditions.checkNotNull(vertex, "Invalid vertex in task based affinity " + taskAffinity 
+        Objects.requireNonNull(vertex, "Invalid vertex in task based affinity " + taskAffinity
             + " for attempt: " + taskAttempt.getID());
         int taskIndex = taskAffinity.getTaskIndex(); 
         Preconditions.checkState(taskIndex >=0 && taskIndex < vertex.getTotalTasks(), 
@@ -491,7 +490,8 @@ public class TaskSchedulerManager extends AbstractService implements
             + " for attempt: " + taskAttempt.getID());
         TaskAttempt affinityAttempt = vertex.getTask(taskIndex).getSuccessfulAttempt();
         if (affinityAttempt != null) {
-          Preconditions.checkNotNull(affinityAttempt.getAssignedContainerID(), affinityAttempt.getID());
+          Objects.requireNonNull(affinityAttempt.getAssignedContainerID(),
+              affinityAttempt.getID() == null ? null : affinityAttempt.getID().toString());
           try {
             taskSchedulers[event.getSchedulerId()].allocateTask(taskAttempt,
                 event.getCapability(),
@@ -641,7 +641,9 @@ public class TaskSchedulerManager extends AbstractService implements
   
   @Override
   public synchronized void serviceStart() throws Exception {
-    InetSocketAddress serviceAddr = clientService.getBindAddress();
+    // clientService is null in case of LocalDAGAppMaster
+    InetSocketAddress serviceAddr = clientService == null ? new InetSocketAddress("127.0.0.1", 0)
+        : clientService.getBindAddress();
     dagAppMaster = appContext.getAppMaster();
     // if web service is enabled then set tracking url. else disable it (value = "").
     // the actual url set on the rm web ui will be the proxy url set by WebAppProxyServlet, which
@@ -859,9 +861,7 @@ public class TaskSchedulerManager extends AbstractService implements
         }
       }
     }
-    if(LOG.isDebugEnabled()) {
-      LOG.debug("Setting job diagnostics to " + sb.toString());
-    }
+    LOG.debug("Setting job diagnostics to {}", sb);
 
     // if history url is set use the same, if historyUrl is set to "" then rm ui disables the
     // history url
