@@ -22,7 +22,7 @@ import static org.apache.hadoop.yarn.util.StringHelper.pajoin;
 
 import java.net.InetSocketAddress;
 
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.inject.name.Names;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -119,9 +119,7 @@ public class WebUIService extends AbstractService {
   @Override
   protected void serviceStop() throws Exception {
     if (this.webApp != null) {
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Stopping WebApp");
-      }
+      LOG.debug("Stopping WebApp");
       this.webApp.stop();
     }
     super.serviceStop();
