@@ -405,20 +405,17 @@ public class RecoveryService extends AbstractService {
   protected void handleSummaryEvent(TezDAGID dagID,
       HistoryEventType eventType,
       SummaryEvent summaryEvent) throws IOException {
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("Handling summary event"
-          + ", dagID=" + dagID
-          + ", eventType=" + eventType);
-    }
+      LOG.debug("Handling summary event, dagID={}, eventType={}", dagID, eventType);
+
     if (summaryStream == null) {
       Path summaryPath = TezCommonUtils.getSummaryRecoveryPath(recoveryPath);
       if (LOG.isDebugEnabled()) {
         LOG.debug("AppId :" + appContext.getApplicationID() + " summaryPath " + summaryPath);
       }
-      if (!recoveryDirFS.exists(summaryPath)) {
-        summaryStream = recoveryDirFS.create(summaryPath, false,
-            bufferSize);
-      } else {
+      try {
+        summaryStream = recoveryDirFS.create(summaryPath, false, bufferSize);
+      } catch (IOException e) {
+        LOG.error("Error handling summary event, eventType=" + eventType, e);
         createFatalErrorFlagDir();
         return;
       }
@@ -456,26 +453,22 @@ public class RecoveryService extends AbstractService {
     RecoveryStream recoveryStream = outputStreamMap.get(dagID);
     if (recoveryStream == null) {
       Path dagFilePath = TezCommonUtils.getDAGRecoveryPath(recoveryPath, dagID.toString());
-      if (recoveryDirFS.exists(dagFilePath)) {
+
+      try {
+        FSDataOutputStream outputStream = recoveryDirFS.create(dagFilePath, false, bufferSize);
+        LOG.debug("Opened DAG recovery file in create mode, filePath={}", dagFilePath);
+        recoveryStream = new RecoveryStream(outputStream);
+      } catch (IOException ioe) {
+        LOG.error("Error handling history event, eventType=" + eventType, ioe);
         createFatalErrorFlagDir();
         return;
-      } else {
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Opening DAG recovery file in create mode"
-              + ", filePath=" + dagFilePath);
-        }
-        FSDataOutputStream outputStream = recoveryDirFS.create(dagFilePath, false, bufferSize);
-        recoveryStream = new RecoveryStream(outputStream);
       }
       outputStreamMap.put(dagID, recoveryStream);
     }
 
 
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("Writing recovery event to output stream"
-          + ", dagId=" + dagID
-          + ", eventType=" + eventType);
-    }
+    LOG.debug("Writing recovery event to output stream, dagId={}, eventType={}",
+      dagID, eventType);
     ++unflushedEventsCount;
     recoveryStream.codedOutputStream.writeFixed32NoTag(event.getHistoryEvent().getEventType().ordinal());
     event.getHistoryEvent().toProtoStream(recoveryStream.codedOutputStream);
@@ -490,11 +483,9 @@ public class RecoveryService extends AbstractService {
     boolean doFlush = false;
     if (maxUnflushedEvents >=0
         && unflushedEventsCount >= maxUnflushedEvents) {
-      if  (LOG.isDebugEnabled()) {
-        LOG.debug("Max unflushed events count reached. Flushing recovery data"
-            + ", unflushedEventsCount=" + unflushedEventsCount
-            + ", maxUnflushedEvents=" + maxUnflushedEvents);
-      }
+        LOG.debug("Max unflushed events count reached. Flushing recovery data, "
+            + "unflushedEventsCount={}, maxUnflushedEvents={}", unflushedEventsCount,
+            maxUnflushedEvents);
       doFlush = true;
     } else if (flushInterval >= 0
         && ((currentTime - lastFlushTime) >= (flushInterval*1000))) {
