@@ -34,6 +34,8 @@ import org.apache.hadoop.conf.Configuration;
 import org.apache.tez.common.ATSConstants;
 import org.apache.tez.common.TezCommonUtils;
 import org.apache.tez.common.VersionInfo;
+import org.apache.tez.common.counters.AggregateTezCounter;
+import org.apache.tez.common.counters.AggregateTezCounterDelegate;
 import org.apache.tez.common.counters.CounterGroup;
 import org.apache.tez.common.counters.TezCounter;
 import org.apache.tez.common.counters.TezCounters;
@@ -52,7 +54,7 @@ import org.apache.tez.dag.records.TezTaskID;
 import org.codehaus.jettison.json.JSONException;
 import org.codehaus.jettison.json.JSONObject;
 
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 
 public class DAGUtils {
 
@@ -153,6 +155,15 @@ public class DAGUtils {
                     counter.getDisplayName());
             }
             counterMap.put(ATSConstants.COUNTER_VALUE, counter.getValue());
+            if (counter instanceof AggregateTezCounter) {
+              counterMap.put(ATSConstants.COUNTER_INSTANCE_COUNT,
+                  ((AggregateTezCounter)counter).getCount());
+              counterMap.put(ATSConstants.COUNTER_MAX_VALUE,
+                    ((AggregateTezCounter)counter).getMax());
+              counterMap.put(ATSConstants.COUNTER_MIN_VALUE,
+                  ((AggregateTezCounter)counter).getMin());
+
+            }
             counterList.add(counterMap);
           }
         }
