@@ -20,12 +20,15 @@ package org.apache.tez.dag.app;
 
 import java.io.IOException;
 import java.nio.ByteBuffer;
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Random;
 
+import com.google.common.collect.Sets;
+import com.google.protobuf.CodedInputStream;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
@@ -36,6 +39,8 @@ import org.apache.hadoop.yarn.api.records.NodeId;
 import org.apache.hadoop.yarn.util.SystemClock;
 import org.apache.log4j.Level;
 import org.apache.log4j.LogManager;
+import org.apache.tez.dag.api.TaskLocationHint;
+import org.apache.tez.dag.api.VertexLocationHint;
 import org.apache.tez.dag.api.oldrecords.TaskAttemptState;
 import org.apache.tez.dag.api.oldrecords.TaskState;
 import org.apache.tez.dag.api.records.DAGProtos.DAGPlan;
@@ -59,6 +64,7 @@ import org.apache.tez.dag.history.events.TaskAttemptStartedEvent;
 import org.apache.tez.dag.history.events.TaskFinishedEvent;
 import org.apache.tez.dag.history.events.TaskStartedEvent;
 import org.apache.tez.dag.history.events.VertexCommitStartedEvent;
+import org.apache.tez.dag.history.events.VertexConfigurationDoneEvent;
 import org.apache.tez.dag.history.events.VertexFinishedEvent;
 import org.apache.tez.dag.history.events.VertexGroupCommitFinishedEvent;
 import org.apache.tez.dag.history.events.VertexGroupCommitStartedEvent;
@@ -91,6 +97,8 @@ public class TestRecoveryParser {
   private Path recoveryPath;
   private DAGAppMaster mockAppMaster;
   private DAGImpl mockDAGImpl;
+  // Protobuf message limit is 64 MB by default
+  private static final int PROTOBUF_DEFAULT_SIZE_LIMIT = 64 << 20;
 
   @Before
   public void setUp() throws IllegalArgumentException, IOException {
@@ -105,7 +113,6 @@ public class TestRecoveryParser {
     mockDAGImpl = mock(DAGImpl.class);
     when(mockAppMaster.createDAG(any(DAGPlan.class), any(TezDAGID.class))).thenReturn(mockDAGImpl);
     parser = new RecoveryParser(mockAppMaster, localFS, recoveryPath, 3);
-    LogManager.getRootLogger().setLevel(Level.DEBUG);
   }
 
   private DAGSummaryData createDAGSummaryData(TezDAGID dagId, boolean completed) {
@@ -169,7 +176,7 @@ public class TestRecoveryParser {
     rService.start();
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGInitializedEvent(dagID, 1L, "user", dagPlan.getName(), null)));
     // only for testing, DAGCommitStartedEvent is not supposed to happen at this time.
@@ -215,7 +222,7 @@ public class TestRecoveryParser {
     rService.start();
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGInitializedEvent(dagID, 1L, "user", dagPlan.getName(), null)));
     rService.handle(new DAGHistoryEvent(dagID,
@@ -264,10 +271,10 @@ public class TestRecoveryParser {
     rService.start();
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // wait until DAGSubmittedEvent is handled in the RecoveryEventHandling thread
     rService.await();
-    rService.outputStreamMap.get(dagID).writeUTF("INVALID_DATA");
+    rService.outputStreamMap.get(dagID).write("INVALID_DATA".getBytes("UTF-8"));
     rService.stop();
 
     // write data in attempt_2
@@ -278,7 +285,7 @@ public class TestRecoveryParser {
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGInitializedEvent(dagID, 1L, "user", dagPlan.getName(), null)));
     rService.await();
-    rService.outputStreamMap.get(dagID).writeUTF("INVALID_DATA");
+    rService.outputStreamMap.get(dagID).write("INVALID_DATA".getBytes("UTF-8"));
     rService.stop();
 
     // corrupted last records will be skipped but the whole recovery logs will be read
@@ -310,7 +317,7 @@ public class TestRecoveryParser {
     // write a DAGSubmittedEvent first to initialize summaryStream
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // write an corrupted SummaryEvent
     rService.summaryStream.writeChars("INVALID_DATA");
     rService.stop();
@@ -344,7 +351,7 @@ public class TestRecoveryParser {
     // write a DAGSubmittedEvent first to initialize summaryStream
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // It should be fine to skip other events, just for testing.
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGCommitStartedEvent(dagID, 0L)));
@@ -376,7 +383,7 @@ public class TestRecoveryParser {
     // write a DAGSubmittedEvent first to initialize summaryStream
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // It should be fine to skip other events, just for testing.
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGCommitStartedEvent(dagID, 0L)));
@@ -412,7 +419,7 @@ public class TestRecoveryParser {
     // write a DAGSubmittedEvent first to initialize summaryStream
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // It should be fine to skip other events, just for testing.
     rService.handle(new DAGHistoryEvent(dagID,
         new VertexCommitStartedEvent(TezVertexID.getInstance(dagID, 0), 0L)));
@@ -445,7 +452,7 @@ public class TestRecoveryParser {
     // write a DAGSubmittedEvent first to initialize summaryStream
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // It should be fine to skip other events, just for testing.
     TezVertexID vertexId = TezVertexID.getInstance(dagID, 0);
     rService.handle(new DAGHistoryEvent(dagID,
@@ -453,7 +460,7 @@ public class TestRecoveryParser {
     rService.handle(new DAGHistoryEvent(dagID,
         new VertexFinishedEvent(vertexId, "v1", 10, 0L, 0L, 
             0L, 0L, 0L, VertexState.SUCCEEDED, 
-            "", null, null, null)));
+            "", null, null, null, null)));
     rService.stop();
 
     DAGRecoveryData dagData = parser.parseRecoveryData();
@@ -482,7 +489,7 @@ public class TestRecoveryParser {
     // write a DAGSubmittedEvent first to initialize summaryStream
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // It should be fine to skip other events, just for testing.
     rService.handle(new DAGHistoryEvent(dagID,
         new VertexGroupCommitStartedEvent(dagID, "group_1", 
@@ -516,7 +523,7 @@ public class TestRecoveryParser {
     // write a DAGSubmittedEvent first to initialize summaryStream
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // It should be fine to skip other events, just for testing.
     TezVertexID v0 = TezVertexID.getInstance(dagID, 0);
     TezVertexID v1 = TezVertexID.getInstance(dagID, 1);
@@ -531,11 +538,11 @@ public class TestRecoveryParser {
     rService.handle(new DAGHistoryEvent(dagID,
         new VertexFinishedEvent(v0, "v1", 10, 0L, 0L, 
             0L, 0L, 0L, VertexState.SUCCEEDED, 
-            "", null, null, null)));
+            "", null, null, null, null)));
     rService.handle(new DAGHistoryEvent(dagID,
         new VertexFinishedEvent(v1, "v1", 10, 0L, 0L, 
             0L, 0L, 0L, VertexState.SUCCEEDED, 
-            "", null, null, null)));
+            "", null, null, null, null)));
     rService.stop();
     
     DAGRecoveryData dagData = parser.parseRecoveryData();
@@ -565,7 +572,7 @@ public class TestRecoveryParser {
     // write a DAGSubmittedEvent first to initialize summaryStream
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // It should be fine to skip other events, just for testing.
     TezVertexID vertexId = TezVertexID.getInstance(dagID, 0);
     rService.handle(new DAGHistoryEvent(dagID,
@@ -573,7 +580,7 @@ public class TestRecoveryParser {
     rService.handle(new DAGHistoryEvent(dagID,
         new VertexFinishedEvent(vertexId, "v1", 10, 0L, 0L, 
             0L, 0L, 0L, VertexState.SUCCEEDED, 
-            "", null, null, null)));
+            "", null, null, null, null)));
     rService.stop();
 
     DAGRecoveryData dagData = parser.parseRecoveryData();
@@ -601,7 +608,7 @@ public class TestRecoveryParser {
     // write a DAGSubmittedEvent first to initialize summaryStream
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     // It should be fine to skip other events, just for testing.
     TezVertexID vertexId = TezVertexID.getInstance(dagID, 0);
     rService.handle(new DAGHistoryEvent(dagID,
@@ -618,6 +625,75 @@ public class TestRecoveryParser {
               + ", but its full recovery events are not seen"));
   }
 
+  @Test(timeout=20000)
+  public void testRecoveryLargeEventData() throws IOException {
+    ApplicationId appId = ApplicationId.newInstance(System.currentTimeMillis(), 1);
+    TezDAGID dagID = TezDAGID.getInstance(appId, 1);
+    AppContext appContext = mock(AppContext.class);
+    when(appContext.getCurrentRecoveryDir()).thenReturn(new Path(recoveryPath+"/1"));
+    when(appContext.getClock()).thenReturn(new SystemClock());
+    when(mockDAGImpl.getID()).thenReturn(dagID);
+    when(appContext.getHadoopShim()).thenReturn(new DefaultHadoopShim());
+    when(appContext.getApplicationID()).thenReturn(appId);
+
+    RecoveryService rService = new RecoveryService(appContext);
+    Configuration conf = new Configuration();
+    conf.setBoolean(RecoveryService.TEZ_TEST_RECOVERY_DRAIN_EVENTS_WHEN_STOPPED, true);
+    rService.init(conf);
+    rService.start();
+
+    DAGPlan dagPlan = TestDAGImpl.createTestDAGPlan();
+    // DAG  DAGSubmittedEvent -> DAGInitializedEvent -> DAGStartedEvent
+    rService.handle(new DAGHistoryEvent(dagID,
+        new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
+            null, "user", new Configuration(), null, null)));
+    DAGInitializedEvent dagInitedEvent = new DAGInitializedEvent(dagID, 100L,
+        "user", "dagName", null);
+    DAGStartedEvent dagStartedEvent = new DAGStartedEvent(dagID, 0L, "user", "dagName");
+    rService.handle(new DAGHistoryEvent(dagID, dagInitedEvent));
+    rService.handle(new DAGHistoryEvent(dagID, dagStartedEvent));
+
+    // Create a Recovery event larger than 64 MB to verify default max protobuf size
+    ArrayList<TaskLocationHint> taskLocationHints = new ArrayList<>(100000);
+    TaskLocationHint taskLocationHint = TaskLocationHint.createTaskLocationHint(
+        Sets.newHashSet("aaaaaaaaaaaaaaa.aaaaaaaaaaaaaaa.aaaaaaaaaaaaaaa",
+            "bbbbbbbbbbbbbbb.bbbbbbbbbbbbbbb.bbbbbbbbbbbbbbb",
+            "ccccccccccccccc.ccccccccccccccc.ccccccccccccccc",
+            "ddddddddddddddd.ddddddddddddddd.ddddddddddddddd",
+            "eeeeeeeeeeeeeee.eeeeeeeeeeeeeee.eeeeeeeeeeeeeee",
+            "fffffffffffffff.fffffffffffffff.fffffffffffffff",
+            "ggggggggggggggg.ggggggggggggggg.ggggggggggggggg",
+            "hhhhhhhhhhhhhhh.hhhhhhhhhhhhhhh.hhhhhhhhhhhhhhh",
+            "iiiiiiiiiiiiiii.iiiiiiiiiiiiiii.iiiiiiiiiiiiiii",
+            "jjjjjjjjjjjjjjj.jjjjjjjjjjjjjjj.jjjjjjjjjjjjjjj",
+            "kkkkkkkkkkkkkkk.kkkkkkkkkkkkkkk.kkkkkkkkkkkkkkk",
+            "lllllllllllllll.lllllllllllllll.lllllllllllllll",
+            "mmmmmmmmmmmmmmm.mmmmmmmmmmmmmmm.mmmmmmmmmmmmmmm",
+            "nnnnnnnnnnnnnnn.nnnnnnnnnnnnnnn.nnnnnnnnnnnnnnn"),
+        Sets.newHashSet("rack1", "rack2", "rack3"));
+    for (int i = 0; i < 100000; i++) {
+      taskLocationHints.add(taskLocationHint);
+    }
+
+    TezVertexID v0Id = TezVertexID.getInstance(dagID, 0);
+    VertexLocationHint vertexLocationHint = VertexLocationHint.create(taskLocationHints);
+    VertexConfigurationDoneEvent vertexConfigurationDoneEvent = new VertexConfigurationDoneEvent(
+        v0Id, 0, 100000, vertexLocationHint, null, null, false);
+    // Verify large protobuf message
+    assertTrue(vertexConfigurationDoneEvent.toProto().getSerializedSize() > PROTOBUF_DEFAULT_SIZE_LIMIT );
+    rService.handle(new DAGHistoryEvent(dagID, vertexConfigurationDoneEvent));
+    rService.stop();
+
+    DAGRecoveryData dagData = parser.parseRecoveryData();
+    VertexRecoveryData v0data = dagData.getVertexRecoveryData(v0Id);
+    assertNotNull("Vertex Recovery Data should be non-null", v0data);
+    VertexConfigurationDoneEvent parsedVertexConfigurationDoneEvent = v0data.getVertexConfigurationDoneEvent();
+    assertNotNull("Vertex Configuration Done Event should be non-null", parsedVertexConfigurationDoneEvent);
+    VertexLocationHint parsedVertexLocationHint = parsedVertexConfigurationDoneEvent.getVertexLocationHint();
+    assertNotNull("Vertex Location Hint should be non-null", parsedVertexLocationHint);
+    assertEquals(parsedVertexLocationHint.getTaskLocationHints().size(), 100000);
+  }
+
   @Test(timeout=5000)
   public void testRecoveryData() throws IOException {
     ApplicationId appId = ApplicationId.newInstance(System.currentTimeMillis(), 1);
@@ -640,7 +716,7 @@ public class TestRecoveryParser {
     // DAG  DAGSubmittedEvent -> DAGInitializedEvent -> DAGStartedEvent
     rService.handle(new DAGHistoryEvent(dagID,
         new DAGSubmittedEvent(dagID, 1L, dagPlan, ApplicationAttemptId.newInstance(appId, 1),
-            null, "user", new Configuration(), null)));
+            null, "user", new Configuration(), null, null)));
     DAGInitializedEvent dagInitedEvent = new DAGInitializedEvent(dagID, 100L, 
         "user", "dagName", null);
     DAGStartedEvent dagStartedEvent = new DAGStartedEvent(dagID, 0L, "user", "dagName");
@@ -652,18 +728,19 @@ public class TestRecoveryParser {
     TezVertexID v1Id = TezVertexID.getInstance(dagID, 1);
     TezVertexID v2Id = TezVertexID.getInstance(dagID, 2);
     // v0 VertexInitializedEvent
-    VertexInitializedEvent v0InitedEvent =  new VertexInitializedEvent(v0Id, "v0", 200L, 400L, 2, null, null, null);
+    VertexInitializedEvent v0InitedEvent =  new VertexInitializedEvent(
+        v0Id, "v0", 200L, 400L, 2, null, null, null, null);
     rService.handle(new DAGHistoryEvent(dagID, v0InitedEvent));
     // v1 VertexFinishedEvent(KILLED)
     VertexFinishedEvent v1FinishedEvent = new VertexFinishedEvent(v1Id, "v1", 2, 300L, 400L, 
         500L, 600L, 700L, VertexState.KILLED, 
-        "", null, null, null);
+        "", null, null, null, null);
     rService.handle(new DAGHistoryEvent(dagID, v1FinishedEvent));
     // v2 VertexInitializedEvent -> VertexStartedEvent
     List<TezEvent> initGeneratedEvents = Lists.newArrayList(
         new TezEvent(DataMovementEvent.create(ByteBuffer.wrap(new byte[0])), null));
     VertexInitializedEvent v2InitedEvent = new VertexInitializedEvent(v2Id, "v2", 200L, 300L,
-        2, null, null, initGeneratedEvents);
+        2, null, null, initGeneratedEvents, null);
     VertexStartedEvent v2StartedEvent = new VertexStartedEvent(v2Id, 0L, 0L);
     rService.handle(new DAGHistoryEvent(dagID, v2InitedEvent));
     rService.handle(new DAGHistoryEvent(dagID, v2StartedEvent));
@@ -702,8 +779,8 @@ public class TestRecoveryParser {
     rService.handle(new DAGHistoryEvent(dagID, ta0t2v2StartedEvent));
     TaskAttemptFinishedEvent ta0t2v2FinishedEvent = new TaskAttemptFinishedEvent(
         ta0t2v2Id, "v1", 500L, 600L, 
-        TaskAttemptState.SUCCEEDED, null, "", null, 
-        null, null, 0L, null, 0L);
+        TaskAttemptState.SUCCEEDED, null, null, "", null,
+        null, null, 0L, null, 0L, null, null, null, null, null);
     rService.handle(new DAGHistoryEvent(dagID, ta0t2v2FinishedEvent));
 
     rService.stop();
