@@ -35,11 +35,13 @@ import org.apache.hadoop.yarn.api.records.Priority;
 import org.apache.hadoop.yarn.api.records.Resource;
 import org.apache.hadoop.yarn.client.api.async.AMRMClientAsync;
 import org.apache.hadoop.yarn.client.api.impl.AMRMClientImpl;
+import org.apache.hadoop.yarn.exceptions.YarnException;
 import org.apache.hadoop.yarn.proto.YarnServiceProtos.SchedulerResourceTypes;
 import org.apache.hadoop.yarn.util.resource.Resources;
 import org.apache.tez.common.MockDNSToSwitchMapping;
 import org.apache.tez.dag.api.TezConfiguration;
 import org.apache.tez.dag.app.MockClock;
+import org.apache.tez.dag.app.dag.Task;
 import org.apache.tez.dag.app.dag.TaskAttempt;
 import org.apache.tez.dag.app.rm.DagAwareYarnTaskScheduler.AMRMClientAsyncWrapper;
 import org.apache.tez.dag.app.rm.DagAwareYarnTaskScheduler.HeldContainer;
@@ -350,7 +352,9 @@ public class TestDagAwareYarnTaskScheduler {
     NodeId speculativeNodeId = mock(NodeId.class);
     when(speculativeNodeId.getHost()).thenReturn(speculativeNode);
     TaskAttempt mockTask5 = mock(TaskAttempt.class);
-    when(mockTask5.getNodesWithSiblingRunningAttempts()).thenReturn(Sets.newHashSet(speculativeNodeId));
+    Task task = mock(Task.class);
+    when(mockTask5.getTask()).thenReturn(task);
+    when(task.getNodesWithRunningAttempts()).thenReturn(Sets.newHashSet(speculativeNodeId));
     Object mockCookie5 = new Object();
     scheduler.allocateTask(mockTask5, mockCapability, hosts, racks,
         mockPriority, null, mockCookie5);
@@ -1527,6 +1531,26 @@ public class TestDagAwareYarnTaskScheduler {
     verify(mockRMClient).removeContainerRequest(reqv0t0);
   }
 
+  @Test
+  public void testMinMaxContainerIdleMillisAreEqual() throws Exception {
+    AMRMClientAsyncWrapperForTest mockRMClient = new AMRMClientAsyncWrapperForTest();
+    Configuration conf = new Configuration();
+    conf.setLong(TezConfiguration.TEZ_AM_CONTAINER_IDLE_RELEASE_TIMEOUT_MIN_MILLIS, 10000);
+    conf.setLong(TezConfiguration.TEZ_AM_CONTAINER_IDLE_RELEASE_TIMEOUT_MAX_MILLIS, 10000);
+
+    TaskSchedulerContext mockApp = setupMockTaskSchedulerContext("host", 0, "url", conf);
+    TaskSchedulerContextDrainable drainableAppCallback = createDrainableContext(mockApp);
+    MockClock clock = new MockClock(1000);
+    NewTaskSchedulerForTest scheduler = new NewTaskSchedulerForTest(drainableAppCallback, mockRMClient, clock);
+    scheduler.initialize();
+
+    NodeId host1 = NodeId.newInstance("host1", 1);
+    Container container1 = Container.newInstance(null, host1, null, null, null, null);
+    HeldContainer heldContainer = scheduler.new HeldContainer(container1);
+    long now = clock.getTime();
+    assertEquals(now + 10000, heldContainer.getIdleExpirationTimestamp(now));
+  }
+
   static class AMRMClientAsyncWrapperForTest extends AMRMClientAsyncWrapper {
     AMRMClientAsyncWrapperForTest() {
       super(new MockAMRMClient(), 10000, null);
@@ -1536,6 +1560,12 @@ public class TestDagAwareYarnTaskScheduler {
       return ((MockAMRMClient) client).getRegistrationResponse();
     }
 
+    @Override
+    public RegisterApplicationMasterResponse registerApplicationMaster(String appHostName, int appHostPort,
+        String appTrackingUrl) throws YarnException, IOException {
+      return client.registerApplicationMaster(appHostName, appHostPort, appTrackingUrl);
+    }
+
     @Override
     protected void serviceStart() {
     }
@@ -1562,10 +1592,9 @@ public class TestDagAwareYarnTaskScheduler {
     protected void serviceStop() {
     }
 
-    @SuppressWarnings("unchecked")
     @Override
-    public RegisterApplicationMasterResponse registerApplicationMaster(
-        String appHostName, int appHostPort, String appTrackingUrl) {
+    public RegisterApplicationMasterResponse registerApplicationMaster(String appHostName, int appHostPort,
+        String appTrackingUrl) {
       mockRegResponse = mock(RegisterApplicationMasterResponse.class);
       Resource mockMaxResource = Resources.createResource(1024*1024, 1024);
       Map<ApplicationAccessType, String> mockAcls = Collections.emptyMap();
