@@ -27,9 +27,10 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 
 import com.google.common.base.Function;
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Lists;
 import com.google.protobuf.ByteString;
@@ -165,7 +166,7 @@ public class MRInputHelpers {
       throws IOException {
     // This may not need to use serialization factory, since OldFormat
     // always uses Writable to write splits.
-    Preconditions.checkNotNull(splitProto, "splitProto cannot be null");
+    Objects.requireNonNull(splitProto, "splitProto cannot be null");
     String className = splitProto.getSplitClassName();
     Class<InputSplit> clazz;
 
@@ -198,7 +199,7 @@ public class MRInputHelpers {
   public static org.apache.hadoop.mapreduce.InputSplit createNewFormatSplitFromUserPayload(
       MRRuntimeProtos.MRSplitProto splitProto, SerializationFactory serializationFactory)
       throws IOException {
-    Preconditions.checkNotNull(splitProto, "splitProto must be specified");
+    Objects.requireNonNull(splitProto, "splitProto must be specified");
     String className = splitProto.getSplitClassName();
     Class<org.apache.hadoop.mapreduce.InputSplit> clazz;
 
@@ -321,18 +322,14 @@ public class MRInputHelpers {
     InputSplitInfoMem splitInfoMem = null;
     JobConf jobConf = new JobConf(conf);
     if (jobConf.getUseNewMapper()) {
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Generating mapreduce api input splits");
-      }
+      LOG.debug("Generating mapreduce api input splits");
       Job job = Job.getInstance(conf);
       org.apache.hadoop.mapreduce.InputSplit[] splits =
           generateNewSplits(job, groupSplits, sortSplits, targetTasks);
       splitInfoMem = new InputSplitInfoMem(splits, createTaskLocationHintsFromSplits(splits),
           splits.length, job.getCredentials(), job.getConfiguration());
     } else {
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Generating mapred api input splits");
-      }
+      LOG.debug("Generating mapred api input splits");
       org.apache.hadoop.mapred.InputSplit[] splits =
           generateOldSplits(jobConf, groupSplits, sortSplits, targetTasks);
       splitInfoMem = new InputSplitInfoMem(splits, createTaskLocationHintsFromSplits(splits),
@@ -771,8 +768,8 @@ public class MRInputHelpers {
 
 
   private static String getStringProperty(Configuration conf, String propertyName) {
-    Preconditions.checkNotNull(conf, "Configuration must be provided");
-    Preconditions.checkNotNull(propertyName, "Property name must be provided");
+    Objects.requireNonNull(conf, "Configuration must be provided");
+    Objects.requireNonNull(propertyName, "Property name must be provided");
     return conf.get(propertyName);
   }
 
@@ -781,7 +778,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getDagIdentifier}
+   * @see InputContext#getDagIdentifier()
    * @param conf configuration instance
    * @return dag index
    */
@@ -801,7 +798,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * * @see {@link InputContext#getTaskVertexIndex}
+   * @see InputContext#getTaskVertexIndex
    * @param conf configuration instance
    * @return vertex index
    */
@@ -821,7 +818,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getTaskIndex}
+   * @see InputContext#getTaskIndex
    * @param conf configuration instance
    * @return task index
    */
@@ -841,7 +838,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getTaskAttemptNumber}
+   * @see InputContext#getTaskAttemptNumber
    * @param conf configuration instance
    * @return task attempt index
    */
@@ -861,7 +858,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getInputIndex}
+   * @see InputContext#getInputIndex
    * @param conf configuration instance
    * @return input index
    */
@@ -871,7 +868,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getDAGName}
+   * @see InputContext#getDAGName
    * @param conf configuration instance
    * @return dag name
    */
@@ -881,7 +878,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getTaskVertexName}
+   * @see InputContext#getTaskVertexName
    * @param conf configuration instance
    * @return vertex name
    */
@@ -891,7 +888,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getSourceVertexName}
+   * @see InputContext#getSourceVertexName
    * @param conf configuration instance
    * @return source name
    */
@@ -901,7 +898,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getApplicationId}
+   * @see InputContext#getApplicationId
    * @param conf configuration instance
    * @return applicationId as a string
    */
@@ -911,7 +908,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getUniqueIdentifier}
+   * @see InputContext#getUniqueIdentifier
    * @param conf configuration instance
    * @return unique identifier for the input
    */
@@ -921,7 +918,7 @@ public class MRInputHelpers {
   }
 
   /**
-   * @see {@link InputContext#getDAGAttemptNumber}
+   * @see InputContext#getDAGAttemptNumber
    * @param conf configuration instance
    * @return attempt number
    */
