@@ -21,17 +21,17 @@ package org.apache.tez.auxservices;
 import org.apache.hadoop.util.DiskChecker;
 import static org.fusesource.leveldbjni.JniDBFactory.asString;
 import static org.fusesource.leveldbjni.JniDBFactory.bytes;
-import static org.jboss.netty.buffer.ChannelBuffers.wrappedBuffer;
-import static org.jboss.netty.handler.codec.http.HttpHeaders.Names.CONTENT_TYPE;
-import static org.jboss.netty.handler.codec.http.HttpMethod.GET;
-import static org.jboss.netty.handler.codec.http.HttpResponseStatus.BAD_REQUEST;
-import static org.jboss.netty.handler.codec.http.HttpResponseStatus.FORBIDDEN;
-import static org.jboss.netty.handler.codec.http.HttpResponseStatus.INTERNAL_SERVER_ERROR;
-import static org.jboss.netty.handler.codec.http.HttpResponseStatus.METHOD_NOT_ALLOWED;
-import static org.jboss.netty.handler.codec.http.HttpResponseStatus.NOT_FOUND;
-import static org.jboss.netty.handler.codec.http.HttpResponseStatus.OK;
-import static org.jboss.netty.handler.codec.http.HttpResponseStatus.UNAUTHORIZED;
-import static org.jboss.netty.handler.codec.http.HttpVersion.HTTP_1_1;
+import static io.netty.buffer.Unpooled.wrappedBuffer;
+import static io.netty.handler.codec.http.HttpHeaders.Names.CONTENT_TYPE;
+import static io.netty.handler.codec.http.HttpMethod.GET;
+import static io.netty.handler.codec.http.HttpResponseStatus.BAD_REQUEST;
+import static io.netty.handler.codec.http.HttpResponseStatus.FORBIDDEN;
+import static io.netty.handler.codec.http.HttpResponseStatus.INTERNAL_SERVER_ERROR;
+import static io.netty.handler.codec.http.HttpResponseStatus.METHOD_NOT_ALLOWED;
+import static io.netty.handler.codec.http.HttpResponseStatus.NOT_FOUND;
+import static io.netty.handler.codec.http.HttpResponseStatus.OK;
+import static io.netty.handler.codec.http.HttpResponseStatus.UNAUTHORIZED;
+import static io.netty.handler.codec.http.HttpVersion.HTTP_1_1;
 
 import java.io.File;
 import java.io.FileNotFoundException;
@@ -48,7 +48,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutionException;
-import java.util.concurrent.Executors;
+import java.util.concurrent.ThreadFactory;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.regex.Pattern;
@@ -71,6 +71,7 @@ import org.apache.tez.common.security.JobTokenIdentifier;
 import org.apache.tez.common.security.JobTokenSecretManager;
 import org.apache.tez.runtime.library.common.Constants;
 import org.apache.tez.runtime.library.common.security.SecureShuffleUtils;
+import org.apache.tez.runtime.library.common.shuffle.api.ShuffleHandlerError;
 import org.apache.tez.runtime.library.common.shuffle.orderedgrouped.ShuffleHeader;
 import org.apache.hadoop.metrics2.MetricsSystem;
 import org.apache.hadoop.metrics2.annotation.Metric;
@@ -84,6 +85,7 @@ import org.apache.hadoop.security.ssl.SSLFactory;
 import org.apache.hadoop.security.token.Token;
 import org.apache.tez.runtime.library.common.sort.impl.TezIndexRecord;
 import org.apache.hadoop.util.Shell;
+import org.apache.hadoop.util.DiskChecker.DiskErrorException;
 import org.apache.hadoop.yarn.api.records.ApplicationId;
 import org.apache.hadoop.yarn.conf.YarnConfiguration;
 import org.apache.hadoop.yarn.proto.YarnServerCommonProtos.VersionProto;
@@ -100,46 +102,47 @@ import org.iq80.leveldb.DB;
 import org.iq80.leveldb.DBException;
 import org.iq80.leveldb.Logger;
 import org.iq80.leveldb.Options;
-import org.jboss.netty.bootstrap.ServerBootstrap;
-import org.jboss.netty.buffer.ChannelBuffers;
-import org.jboss.netty.channel.Channel;
-import org.jboss.netty.channel.ChannelFactory;
-import org.jboss.netty.channel.ChannelFuture;
-import org.jboss.netty.channel.ChannelFutureListener;
-import org.jboss.netty.channel.ChannelHandler;
-import org.jboss.netty.channel.ChannelHandlerContext;
-import org.jboss.netty.channel.ChannelPipeline;
-import org.jboss.netty.channel.ChannelPipelineFactory;
-import org.jboss.netty.channel.ChannelStateEvent;
-import org.jboss.netty.channel.Channels;
-import org.jboss.netty.channel.ExceptionEvent;
-import org.jboss.netty.channel.MessageEvent;
-import org.jboss.netty.channel.SimpleChannelUpstreamHandler;
-import org.jboss.netty.channel.group.ChannelGroup;
-import org.jboss.netty.channel.group.DefaultChannelGroup;
-import org.jboss.netty.channel.socket.nio.NioServerBossPool;
-import org.jboss.netty.channel.socket.nio.NioServerSocketChannelFactory;
-import org.jboss.netty.channel.socket.nio.NioWorkerPool;
-import org.jboss.netty.handler.codec.frame.TooLongFrameException;
-import org.jboss.netty.handler.codec.http.DefaultHttpResponse;
-import org.jboss.netty.handler.codec.http.HttpChunkAggregator;
-import org.jboss.netty.handler.codec.http.HttpHeaders;
-import org.jboss.netty.handler.codec.http.HttpRequest;
-import org.jboss.netty.handler.codec.http.HttpRequestDecoder;
-import org.jboss.netty.handler.codec.http.HttpResponse;
-import org.jboss.netty.handler.codec.http.HttpResponseEncoder;
-import org.jboss.netty.handler.codec.http.HttpResponseStatus;
-import org.jboss.netty.handler.codec.http.QueryStringDecoder;
-import org.jboss.netty.handler.ssl.SslHandler;
-import org.jboss.netty.handler.stream.ChunkedWriteHandler;
-import org.jboss.netty.handler.timeout.IdleState;
-import org.jboss.netty.handler.timeout.IdleStateAwareChannelHandler;
-import org.jboss.netty.handler.timeout.IdleStateEvent;
-import org.jboss.netty.handler.timeout.IdleStateHandler;
-import org.jboss.netty.util.CharsetUtil;
-import org.jboss.netty.util.HashedWheelTimer;
-import org.jboss.netty.util.ThreadNameDeterminer;
-import org.jboss.netty.util.Timer;
+
+import io.netty.bootstrap.ServerBootstrap;
+import io.netty.buffer.ByteBuf;
+import io.netty.buffer.Unpooled;
+import io.netty.channel.Channel;
+import io.netty.channel.ChannelHandler.Sharable;
+import io.netty.channel.ChannelDuplexHandler;
+import io.netty.channel.ChannelFuture;
+import io.netty.channel.ChannelFutureListener;
+import io.netty.channel.ChannelHandlerContext;
+import io.netty.channel.ChannelInboundHandlerAdapter;
+import io.netty.channel.ChannelInitializer;
+import io.netty.channel.ChannelOption;
+import io.netty.channel.ChannelPipeline;
+import io.netty.channel.group.ChannelGroup;
+import io.netty.channel.group.DefaultChannelGroup;
+import io.netty.channel.nio.NioEventLoopGroup;
+import io.netty.channel.socket.nio.NioServerSocketChannel;
+import io.netty.channel.socket.nio.NioSocketChannel;
+import io.netty.handler.codec.TooLongFrameException;
+import io.netty.handler.codec.http.DefaultFullHttpResponse;
+import io.netty.handler.codec.http.DefaultHttpResponse;
+import io.netty.handler.codec.http.FullHttpRequest;
+import io.netty.handler.codec.http.FullHttpResponse;
+import io.netty.handler.codec.http.HttpHeaders;
+import io.netty.handler.codec.http.HttpObjectAggregator;
+import io.netty.handler.codec.http.HttpRequest;
+import io.netty.handler.codec.http.HttpRequestDecoder;
+import io.netty.handler.codec.http.HttpResponse;
+import io.netty.handler.codec.http.HttpResponseEncoder;
+import io.netty.handler.codec.http.HttpResponseStatus;
+import io.netty.handler.codec.http.LastHttpContent;
+import io.netty.handler.codec.http.QueryStringDecoder;
+import io.netty.handler.ssl.SslHandler;
+import io.netty.handler.stream.ChunkedWriteHandler;
+import io.netty.handler.timeout.IdleState;
+import io.netty.handler.timeout.IdleStateEvent;
+import io.netty.handler.timeout.IdleStateHandler;
+import io.netty.util.CharsetUtil;
+import io.netty.util.concurrent.GlobalEventExecutor;
+
 import org.slf4j.LoggerFactory;
 
 import com.google.common.annotations.VisibleForTesting;
@@ -180,11 +183,15 @@ public class ShuffleHandler extends AuxiliaryService {
   private static final String INDEX_FILE_NAME = "file.out.index";
 
   private int port;
-  private ChannelFactory selector;
-  private final ChannelGroup accepted = new DefaultChannelGroup();
-  protected HttpPipelineFactory pipelineFact;
+  private NioEventLoopGroup bossGroup;
+  private NioEventLoopGroup workerGroup;
+  private final ChannelGroup accepted = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);
   private int sslFileBufferSize;
 
+  // pipeline items
+  private Shuffle SHUFFLE;
+  private SSLFactory sslFactory;
+
   /**
    * Should the shuffle use posix_fadvise calls to manage the OS cache during
    * sendfile
@@ -260,7 +267,6 @@ public class ShuffleHandler extends AuxiliaryService {
   boolean connectionKeepAliveEnabled = false;
   private int connectionKeepAliveTimeOut;
   private int mapOutputMetaInfoCacheSize;
-  private Timer timer;
 
   @Metrics(about="Shuffle output metrics", context="mapred", name="tez")
   static class ShuffleMetrics implements ChannelFutureListener {
@@ -297,7 +303,7 @@ public class ShuffleHandler extends AuxiliaryService {
     @Override
     public void operationComplete(ChannelFuture future) throws Exception {
       if (!future.isSuccess()) {
-        future.getChannel().close();
+        future.channel().close();
         return;
       }
       int waitCount = this.reduceContext.getMapsToWait().decrementAndGet();
@@ -305,21 +311,21 @@ public class ShuffleHandler extends AuxiliaryService {
         metrics.operationComplete(future);
         // Let the idle timer handler close keep-alive connections
         if (reduceContext.getKeepAlive()) {
-          ChannelPipeline pipeline = future.getChannel().getPipeline();
+          ChannelPipeline pipeline = future.channel().pipeline();
           TimeoutHandler timeoutHandler =
               (TimeoutHandler) pipeline.get(TIMEOUT_HANDLER);
           timeoutHandler.setEnabledTimeout(true);
         } else {
-          future.getChannel().close();
+          future.channel().close();
         }
       } else {
-        pipelineFact.getSHUFFLE().sendMap(reduceContext);
+        SHUFFLE.sendMap(reduceContext);
       }
     }
   }
 
   /**
-   * Maintain parameters per messageReceived() Netty context.
+   * Maintain parameters per channelRead() Netty context.
    * Allows sendMapOutput calls from operationComplete()
    */
   private static class ReduceContext {
@@ -416,9 +422,11 @@ public class ShuffleHandler extends AuxiliaryService {
    */
   public static ByteBuffer serializeMetaData(int port) throws IOException {
     //TODO these bytes should be versioned
-    DataOutputBuffer port_dob = new DataOutputBuffer();
-    port_dob.writeInt(port);
-    return ByteBuffer.wrap(port_dob.getData(), 0, port_dob.getLength());
+    DataOutputBuffer portDob = new DataOutputBuffer();
+    portDob.writeInt(port);
+    ByteBuffer buf = ByteBuffer.wrap(portDob.getData(), 0, portDob.getLength());
+    portDob.close();
+    return buf;
   }
 
   /**
@@ -431,6 +439,7 @@ public class ShuffleHandler extends AuxiliaryService {
     DataInputByteBuffer in = new DataInputByteBuffer();
     in.reset(meta);
     int port = in.readInt();
+    in.close();
     return port;
   }
 
@@ -513,22 +522,23 @@ public class ShuffleHandler extends AuxiliaryService {
         DEFAULT_SHUFFLE_MAX_SESSION_OPEN_FILES);
 
     final String BOSS_THREAD_NAME_PREFIX = "Tez Shuffle Handler Boss #";
-    NioServerBossPool bossPool = new NioServerBossPool(Executors.newCachedThreadPool(), 1, new ThreadNameDeterminer() {
+    AtomicInteger bossThreadCounter = new AtomicInteger(0);
+    bossGroup = new NioEventLoopGroup(maxShuffleThreads, new ThreadFactory() {
       @Override
-      public String determineThreadName(String currentThreadName, String proposedThreadName) throws Exception {
-        return BOSS_THREAD_NAME_PREFIX + currentThreadName.substring(currentThreadName.lastIndexOf('-') + 1);
+      public Thread newThread(Runnable r) {
+        return new Thread(r, BOSS_THREAD_NAME_PREFIX + bossThreadCounter.incrementAndGet());
       }
     });
 
     final String WORKER_THREAD_NAME_PREFIX = "Tez Shuffle Handler Worker #";
-    NioWorkerPool workerPool = new NioWorkerPool(Executors.newCachedThreadPool(), maxShuffleThreads, new ThreadNameDeterminer() {
+    AtomicInteger workerThreadCounter = new AtomicInteger(0);
+    workerGroup = new NioEventLoopGroup(maxShuffleThreads, new ThreadFactory() {
       @Override
-      public String determineThreadName(String currentThreadName, String proposedThreadName) throws Exception {
-        return WORKER_THREAD_NAME_PREFIX + currentThreadName.substring(currentThreadName.lastIndexOf('-') + 1);
+      public Thread newThread(Runnable r) {
+        return new Thread(r, WORKER_THREAD_NAME_PREFIX + workerThreadCounter.incrementAndGet());
       }
     });
 
-    selector = new NioServerSocketChannelFactory(bossPool, workerPool);
     super.serviceInit(new YarnConfiguration(conf));
   }
 
@@ -539,25 +549,24 @@ public class ShuffleHandler extends AuxiliaryService {
     userRsrc = new ConcurrentHashMap<String,String>();
     secretManager = new JobTokenSecretManager();
     recoverState(conf);
-    ServerBootstrap bootstrap = new ServerBootstrap(selector);
-    // Timer is shared across entire factory and must be released separately
-    timer = new HashedWheelTimer();
-    try {
-      pipelineFact = new HttpPipelineFactory(conf, timer);
-    } catch (Exception ex) {
-      throw new RuntimeException(ex);
-    }
-    bootstrap.setOption("backlog", conf.getInt(SHUFFLE_LISTEN_QUEUE_SIZE,
-        DEFAULT_SHUFFLE_LISTEN_QUEUE_SIZE));
-    bootstrap.setOption("child.keepAlive", true);
-    bootstrap.setPipelineFactory(pipelineFact);
+    ServerBootstrap bootstrap = new ServerBootstrap()
+        .channel(NioServerSocketChannel.class)
+        .group(bossGroup, workerGroup)
+        .localAddress(port)
+        .option(ChannelOption.SO_BACKLOG,
+            conf.getInt(SHUFFLE_LISTEN_QUEUE_SIZE, DEFAULT_SHUFFLE_LISTEN_QUEUE_SIZE))
+        .childOption(ChannelOption.SO_KEEPALIVE, true);
+    initPipeline(bootstrap, conf);
     port = conf.getInt(SHUFFLE_PORT_CONFIG_KEY, DEFAULT_SHUFFLE_PORT);
-    Channel ch = bootstrap.bind(new InetSocketAddress(port));
+    Channel ch = bootstrap.bind().sync().channel();
     accepted.add(ch);
-    port = ((InetSocketAddress)ch.getLocalAddress()).getPort();
+
+    // setup port
+    port = ((InetSocketAddress)ch.localAddress()).getPort();
     conf.set(SHUFFLE_PORT_CONFIG_KEY, Integer.toString(port));
-    pipelineFact.SHUFFLE.setPort(port);
+    SHUFFLE.setPort(port);
     LOG.info(getName() + " listening on port " + port);
+
     super.serviceStart();
 
     sslFileBufferSize = conf.getInt(SUFFLE_SSL_FILE_BUFFER_SIZE_KEY,
@@ -573,20 +582,50 @@ public class ShuffleHandler extends AuxiliaryService {
           DEFAULT_SHUFFLE_MAPOUTPUT_META_INFO_CACHE_SIZE));
   }
 
+  private void initPipeline(ServerBootstrap bootstrap, Configuration conf) throws Exception {
+    SHUFFLE = getShuffle(conf);
+    if (conf.getBoolean(SHUFFLE_SSL_ENABLED_KEY, SHUFFLE_SSL_ENABLED_DEFAULT)) {
+      LOG.info("Encrypted shuffle is enabled.");
+      sslFactory = new SSLFactory(SSLFactory.Mode.SERVER, conf);
+      sslFactory.init();
+    }
+
+    ChannelInitializer<NioSocketChannel> channelInitializer =
+        new ChannelInitializer<NioSocketChannel>() {
+          @Override
+      public void initChannel(NioSocketChannel ch) throws Exception {
+        ChannelPipeline pipeline = ch.pipeline();
+        if (sslFactory != null) {
+          pipeline.addLast("ssl", new SslHandler(sslFactory.createSSLEngine()));
+        }
+        pipeline.addLast("decoder", new HttpRequestDecoder());
+        pipeline.addLast("aggregator", new HttpObjectAggregator(1 << 16));
+        pipeline.addLast("encoder", new HttpResponseEncoder());
+        pipeline.addLast("chunking", new ChunkedWriteHandler());
+        pipeline.addLast("shuffle", SHUFFLE);
+        pipeline.addLast("idle", new IdleStateHandler(0, connectionKeepAliveTimeOut, 0));
+        pipeline.addLast(TIMEOUT_HANDLER, new TimeoutHandler());
+      }
+    };
+    bootstrap.childHandler(channelInitializer);
+  }
+
+  private void destroyPipeline() {
+    if (sslFactory != null) {
+      sslFactory.destroy();
+    }
+  }
+
   @Override
   protected void serviceStop() throws Exception {
     accepted.close().awaitUninterruptibly(10, TimeUnit.SECONDS);
-    if (selector != null) {
-      ServerBootstrap bootstrap = new ServerBootstrap(selector);
-      bootstrap.releaseExternalResources();
+    if (bossGroup != null) {
+      bossGroup.shutdownGracefully();
     }
-    if (pipelineFact != null) {
-      pipelineFact.destroy();
-    }
-    if (timer != null) {
-      // Release this shared timer resource
-      timer.stop();
+    if (workerGroup != null) {
+      workerGroup.shutdownGracefully();
     }
+    destroyPipeline();
     if (stateDb != null) {
       stateDb.close();
     }
@@ -608,6 +647,10 @@ public class ShuffleHandler extends AuxiliaryService {
     return new Shuffle(conf);
   }
 
+  protected JobTokenSecretManager getSecretManager() {
+    return secretManager;
+  }
+
   private void recoverState(Configuration conf) throws IOException {
     Path recoveryRoot = getRecoveryPath();
     if (recoveryRoot != null) {
@@ -727,7 +770,7 @@ public class ShuffleHandler extends AuxiliaryService {
   private void addJobToken(JobID jobId, String user,
       Token<JobTokenIdentifier> jobToken) {
     userRsrc.put(jobId.toString(), user);
-    secretManager.addTokenForJob(jobId.toString(), jobToken);
+    getSecretManager().addTokenForJob(jobId.toString(), jobToken);
     LOG.info("Added token for " + jobId.toString());
   }
 
@@ -772,7 +815,7 @@ public class ShuffleHandler extends AuxiliaryService {
 
   private void removeJobShuffleInfo(JobID jobId) throws IOException {
     String jobIdStr = jobId.toString();
-    secretManager.removeTokenForJob(jobIdStr);
+    getSecretManager().removeTokenForJob(jobIdStr);
     userRsrc.remove(jobIdStr);
     if (stateDb != null) {
       try {
@@ -793,7 +836,7 @@ public class ShuffleHandler extends AuxiliaryService {
     }
   }
 
-  static class TimeoutHandler extends IdleStateAwareChannelHandler {
+  static class TimeoutHandler extends ChannelDuplexHandler {
 
     private boolean enabledTimeout;
 
@@ -802,61 +845,16 @@ public class ShuffleHandler extends AuxiliaryService {
     }
 
     @Override
-    public void channelIdle(ChannelHandlerContext ctx, IdleStateEvent e) {
-      if (e.getState() == IdleState.WRITER_IDLE && enabledTimeout) {
-        e.getChannel().close();
+    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
+      if (evt instanceof IdleStateEvent) {
+        IdleStateEvent e = (IdleStateEvent) evt;
+        if (e.state() == IdleState.WRITER_IDLE && enabledTimeout) {
+          ctx.channel().close();
+        }
       }
     }
   }
 
-  class HttpPipelineFactory implements ChannelPipelineFactory {
-
-    final Shuffle SHUFFLE;
-    private SSLFactory sslFactory;
-    private final ChannelHandler idleStateHandler;
-
-    public HttpPipelineFactory(Configuration conf, Timer timer) throws Exception {
-      SHUFFLE = getShuffle(conf);
-      if (conf.getBoolean(SHUFFLE_SSL_ENABLED_KEY,
-                          SHUFFLE_SSL_ENABLED_DEFAULT)) {
-        LOG.info("Encrypted shuffle is enabled.");
-        sslFactory = new SSLFactory(SSLFactory.Mode.SERVER, conf);
-        sslFactory.init();
-      }
-      this.idleStateHandler = new IdleStateHandler(timer, 0, connectionKeepAliveTimeOut, 0);
-    }
-
-    public Shuffle getSHUFFLE() {
-      return SHUFFLE;
-    }
-
-    public void destroy() {
-      if (sslFactory != null) {
-        sslFactory.destroy();
-      }
-    }
-
-    @Override
-    public ChannelPipeline getPipeline() throws Exception {
-      ChannelPipeline pipeline = Channels.pipeline();
-      if (sslFactory != null) {
-        pipeline.addLast("ssl", new SslHandler(sslFactory.createSSLEngine()));
-      }
-      pipeline.addLast("decoder", new HttpRequestDecoder());
-      pipeline.addLast("aggregator", new HttpChunkAggregator(1 << 16));
-      pipeline.addLast("encoder", new HttpResponseEncoder());
-      pipeline.addLast("chunking", new ChunkedWriteHandler());
-      pipeline.addLast("shuffle", SHUFFLE);
-      pipeline.addLast("idle", idleStateHandler);
-      pipeline.addLast(TIMEOUT_HANDLER, new TimeoutHandler());
-      return pipeline;
-      // TODO factor security manager into pipeline
-      // TODO factor out encode/decode to permit binary shuffle
-      // TODO factor out decode of index to permit alt. models
-    }
-
-  }
-
   protected static class Range {
     final int first;
     final int last;
@@ -880,7 +878,8 @@ public class ShuffleHandler extends AuxiliaryService {
     }
   }
 
-  class Shuffle extends SimpleChannelUpstreamHandler {
+  @Sharable
+  class Shuffle extends ChannelInboundHandlerAdapter {
 
     private static final int MAX_WEIGHT = 10 * 1024 * 1024;
     private static final int EXPIRE_AFTER_ACCESS_MINUTES = 5;
@@ -926,9 +925,7 @@ public class ShuffleHandler extends AuxiliaryService {
           Path mapOutputFileName = lDirAlloc.getLocalPathToRead(
               attemptBase + Path.SEPARATOR + DATA_FILE_NAME, conf);
 
-          if (LOG.isDebugEnabled()) {
-            LOG.debug("Loaded : " + key + " via loader");
-          }
+          LOG.debug("Loaded : {} via loader", key);
           return new AttemptPathInfo(indexFileName, mapOutputFileName);
         }
       });
@@ -968,46 +965,49 @@ public class ShuffleHandler extends AuxiliaryService {
     }
 
     @Override
-    public void channelOpen(ChannelHandlerContext ctx, ChannelStateEvent evt)
+    public void channelActive(ChannelHandlerContext ctx)
         throws Exception {
 
       if ((maxShuffleConnections > 0) && (accepted.size() >= maxShuffleConnections)) {
         LOG.info(String.format("Current number of shuffle connections (%d) is " +
             "greater than or equal to the max allowed shuffle connections (%d)",
             accepted.size(), maxShuffleConnections));
-        evt.getChannel().close();
+        ctx.channel().close();
         return;
       }
-      accepted.add(evt.getChannel());
-      super.channelOpen(ctx, evt);
+      accepted.add(ctx.channel());
+      super.channelActive(ctx);
     }
 
     @Override
-    public void messageReceived(ChannelHandlerContext ctx, MessageEvent evt)
+    public void channelRead(ChannelHandlerContext ctx, Object message)
         throws Exception {
-      HttpRequest request = (HttpRequest) evt.getMessage();
+      FullHttpRequest request = (FullHttpRequest) message;
+      handleRequest(ctx, request);
+      request.release();
+    }
+
+    private void handleRequest(ChannelHandlerContext ctx, FullHttpRequest request)
+        throws IOException, Exception {
       if (request.getMethod() != GET) {
           sendError(ctx, METHOD_NOT_ALLOWED);
           return;
       }
       // Check whether the shuffle version is compatible
       if (!ShuffleHeader.DEFAULT_HTTP_HEADER_NAME.equals(
-          request.getHeader(ShuffleHeader.HTTP_HEADER_NAME))
+          request.headers().get(ShuffleHeader.HTTP_HEADER_NAME))
           || !ShuffleHeader.DEFAULT_HTTP_HEADER_VERSION.equals(
-              request.getHeader(ShuffleHeader.HTTP_HEADER_VERSION))) {
+              request.headers().get(ShuffleHeader.HTTP_HEADER_VERSION))) {
         sendError(ctx, "Incompatible shuffle request version", BAD_REQUEST);
+        return;
       }
-      final Map<String,List<String>> q =
-        new QueryStringDecoder(request.getUri()).getParameters();
+      final Map<String, List<String>> q = new QueryStringDecoder(request.getUri()).parameters();
       final List<String> keepAliveList = q.get("keepAlive");
       final List<String> dagCompletedQ = q.get("dagAction");
       boolean keepAliveParam = false;
       if (keepAliveList != null && keepAliveList.size() == 1) {
         keepAliveParam = Boolean.parseBoolean(keepAliveList.get(0));
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("KeepAliveParam : " + keepAliveList
-            + " : " + keepAliveParam);
-        }
+        LOG.debug("KeepAliveParam : {} : {}", keepAliveList, keepAliveParam);
       }
       final List<String> mapIds = splitMaps(q.get("map"));
       final Range reduceRange = splitReduces(q.get("reduce"));
@@ -1022,7 +1022,7 @@ public class ShuffleHandler extends AuxiliaryService {
             "\n  keepAlive: " + keepAliveParam);
       }
       // If the request is for Dag Deletion, process the request and send OK.
-      if (deleteDagDirectories(evt, dagCompletedQ, jobQ, dagIdQ))  {
+      if (deleteDagDirectories(ctx.channel(), dagCompletedQ, jobQ, dagIdQ))  {
         return;
       }
       if (mapIds == null || reduceRange == null || jobQ == null || dagIdQ == null) {
@@ -1071,8 +1071,8 @@ public class ShuffleHandler extends AuxiliaryService {
 
       Map<String, MapOutputInfo> mapOutputInfoMap =
           new HashMap<String, MapOutputInfo>();
-      Channel ch = evt.getChannel();
-      ChannelPipeline pipeline = ch.getPipeline();
+      Channel ch = ctx.channel();
+      ChannelPipeline pipeline = ch.pipeline();
       TimeoutHandler timeoutHandler = (TimeoutHandler)pipeline.get(TIMEOUT_HANDLER);
       timeoutHandler.setEnabledTimeout(false);
       String user = userRsrc.get(jobId);
@@ -1080,27 +1080,39 @@ public class ShuffleHandler extends AuxiliaryService {
       try {
         populateHeaders(mapIds, jobId, dagId, user, reduceRange,
           response, keepAliveParam, mapOutputInfoMap);
-      } catch(IOException e) {
+      } catch (DiskErrorException e) { // fatal error: fetcher should be aware of that
+        LOG.error("Shuffle error in populating headers (fatal: DiskErrorException):", e);
+        String errorMessage = getErrorMessage(e);
+        // custom message, might be noticed by fetchers
+        // it should reuse the current response object, as headers have been already set for it
+        sendFakeShuffleHeaderWithError(ctx,
+            ShuffleHandlerError.DISK_ERROR_EXCEPTION + ": " + errorMessage, response);
+        return;
+      } catch (IOException e) {
         ch.write(response);
         LOG.error("Shuffle error in populating headers :", e);
         String errorMessage = getErrorMessage(e);
-        sendError(ctx,errorMessage , INTERNAL_SERVER_ERROR);
+        sendError(ctx, errorMessage, INTERNAL_SERVER_ERROR);
         return;
       }
       ch.write(response);
-      //Initialize one ReduceContext object per messageReceived call
+      //Initialize one ReduceContext object per channelRead call
       boolean keepAlive = keepAliveParam || connectionKeepAliveEnabled;
       ReduceContext reduceContext = new ReduceContext(mapIds, reduceRange, ctx,
           user, mapOutputInfoMap, jobId, dagId, keepAlive);
       for (int i = 0; i < Math.min(maxSessionOpenFiles, mapIds.size()); i++) {
         ChannelFuture nextMap = sendMap(reduceContext);
         if(nextMap == null) {
+          // by this special message flushed, we can make sure the whole response is finished
+          ch.writeAndFlush(LastHttpContent.EMPTY_LAST_CONTENT);
           return;
         }
       }
+      // by this special message flushed, we can make sure the whole response is finished
+      ch.writeAndFlush(LastHttpContent.EMPTY_LAST_CONTENT);
     }
 
-    private boolean deleteDagDirectories(MessageEvent evt,
+    private boolean deleteDagDirectories(Channel channel,
                                          List<String> dagCompletedQ, List<String> jobQ,
                                          List<String> dagIdQ) {
       if (jobQ == null || jobQ.isEmpty()) {
@@ -1117,8 +1129,8 @@ public class ShuffleHandler extends AuxiliaryService {
         } catch (IOException e) {
           LOG.warn("Encountered exception during dag delete "+ e);
         }
-        evt.getChannel().write(new DefaultHttpResponse(HTTP_1_1, OK));
-        evt.getChannel().close();
+        channel.writeAndFlush(new DefaultHttpResponse(HTTP_1_1, OK))
+            .addListener(ChannelFutureListener.CLOSE);
         return true;
       }
       return false;
@@ -1126,7 +1138,7 @@ public class ShuffleHandler extends AuxiliaryService {
 
     /**
      * Calls sendMapOutput for the mapId pointed by ReduceContext.mapsToSend
-     * and increments it. This method is first called by messageReceived()
+     * and increments it. This method is first called by channelRead()
      * maxSessionOpenFiles times and then on the completion of every
      * sendMapOutput operation. This limits the number of open files on a node,
      * which can get really large(exhausting file descriptors on the NM) if all
@@ -1136,7 +1148,6 @@ public class ShuffleHandler extends AuxiliaryService {
      */
     public ChannelFuture sendMap(ReduceContext reduceContext)
         throws Exception {
-
       ChannelFuture nextMap = null;
       if (reduceContext.getMapsToSend().get() <
           reduceContext.getMapIds().size()) {
@@ -1145,14 +1156,16 @@ public class ShuffleHandler extends AuxiliaryService {
 
         try {
           MapOutputInfo info = reduceContext.getInfoMap().get(mapId);
+
           if (info == null) {
             info = getMapOutputInfo(reduceContext.dagId, mapId, reduceContext.getReduceRange(),
                 reduceContext.getJobId(),
                 reduceContext.getUser());
           }
+
           nextMap = sendMapOutput(
               reduceContext.getCtx(),
-              reduceContext.getCtx().getChannel(),
+              reduceContext.getCtx().channel(),
               reduceContext.getUser(), mapId,
               reduceContext.getReduceRange(), info);
           if (null == nextMap) {
@@ -1211,11 +1224,8 @@ public class ShuffleHandler extends AuxiliaryService {
         AttemptPathIdentifier identifier = new AttemptPathIdentifier(
             jobId, dagId, user, mapId);
         pathInfo = pathCache.get(identifier);
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Retrieved pathInfo for " + identifier +
-              " check for corresponding loaded messages to determine whether" +
-              " it was loaded or cached");
-        }
+        LOG.debug("Retrieved pathInfo for {} check for corresponding loaded " +
+            "messages to determine whether it was loaded or cached", identifier);
       } catch (ExecutionException e) {
         if (e.getCause() instanceof IOException) {
           throw (IOException) e.getCause();
@@ -1285,17 +1295,13 @@ public class ShuffleHandler extends AuxiliaryService {
 
     protected void setResponseHeaders(HttpResponse response, boolean keepAliveParam, long contentLength) {
       if (connectionKeepAliveEnabled || keepAliveParam) {
-        response.setHeader(HttpHeaders.Names.CONTENT_LENGTH, String.valueOf(contentLength));
-        response.setHeader(HttpHeaders.Names.CONNECTION, HttpHeaders.Values.KEEP_ALIVE);
-        response.setHeader(HttpHeaders.Values.KEEP_ALIVE, "timeout=" + connectionKeepAliveTimeOut);
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Content Length in shuffle : " + contentLength);
-        }
+        response.headers().set(HttpHeaders.Names.CONTENT_LENGTH, String.valueOf(contentLength));
+        response.headers().set(HttpHeaders.Names.CONNECTION, HttpHeaders.Values.KEEP_ALIVE);
+        response.headers().set(HttpHeaders.Values.KEEP_ALIVE, "timeout=" + connectionKeepAliveTimeOut);
+        LOG.debug("Content Length in shuffle : {}", contentLength);
       } else {
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Setting connection close header...");
-        }
-        response.setHeader(HttpHeaders.Names.CONNECTION, CONNECTION_CLOSE);
+        LOG.debug("Setting connection close header...");
+        response.headers().set(HttpHeaders.Names.CONNECTION, CONNECTION_CLOSE);
       }
     }
 
@@ -1337,7 +1343,7 @@ public class ShuffleHandler extends AuxiliaryService {
     protected void verifyRequest(String appid, ChannelHandlerContext ctx,
         HttpRequest request, HttpResponse response, URL requestUri)
         throws IOException {
-      SecretKey tokenSecret = secretManager.retrieveTokenSecret(appid);
+      SecretKey tokenSecret = getSecretManager().retrieveTokenSecret(appid);
       if (null == tokenSecret) {
         LOG.info("Request for unknown token " + appid);
         throw new IOException("could not find jobid");
@@ -1346,7 +1352,7 @@ public class ShuffleHandler extends AuxiliaryService {
       String enc_str = SecureShuffleUtils.buildMsgFrom(requestUri);
       // hash from the fetcher
       String urlHashStr =
-        request.getHeader(SecureShuffleUtils.HTTP_HEADER_URL_HASH);
+        request.headers().get(SecureShuffleUtils.HTTP_HEADER_URL_HASH);
       if (urlHashStr == null) {
         LOG.info("Missing header hash for " + appid);
         throw new IOException("fetcher cannot be authenticated");
@@ -1362,11 +1368,11 @@ public class ShuffleHandler extends AuxiliaryService {
       String reply =
         SecureShuffleUtils.generateHash(urlHashStr.getBytes(Charsets.UTF_8),
             tokenSecret);
-      response.setHeader(SecureShuffleUtils.HTTP_HEADER_REPLY_URL_HASH, reply);
+      response.headers().set(SecureShuffleUtils.HTTP_HEADER_REPLY_URL_HASH, reply);
       // Put shuffle version into http header
-      response.setHeader(ShuffleHeader.HTTP_HEADER_NAME,
+      response.headers().set(ShuffleHeader.HTTP_HEADER_NAME,
           ShuffleHeader.DEFAULT_HTTP_HEADER_NAME);
-      response.setHeader(ShuffleHeader.HTTP_HEADER_VERSION,
+      response.headers().set(ShuffleHeader.HTTP_HEADER_VERSION,
           ShuffleHeader.DEFAULT_HTTP_HEADER_VERSION);
       if (LOG.isDebugEnabled()) {
         int len = reply.length();
@@ -1414,23 +1420,12 @@ public class ShuffleHandler extends AuxiliaryService {
         return null;
       }
       ChannelFuture writeFuture;
-      if (ch.getPipeline().get(SslHandler.class) == null) {
+      if (ch.pipeline().get(SslHandler.class) == null) {
         final FadvisedFileRegion partition = new FadvisedFileRegion(spill,
             rangeOffset, rangePartLength, manageOsCache, readaheadLength,
             readaheadPool, spillFile.getAbsolutePath(),
             shuffleBufferSize, shuffleTransferToAllowed);
         writeFuture = ch.write(partition);
-        writeFuture.addListener(new ChannelFutureListener() {
-            // TODO error handling; distinguish IO/connection failures,
-            //      attribute to appropriate spill output
-          @Override
-          public void operationComplete(ChannelFuture future) {
-            if (future.isSuccess()) {
-              partition.transferSuccessful();
-            }
-            partition.releaseExternalResources();
-          }
-        });
       } else {
         // HTTPS cannot be done with zero copy.
         final FadvisedChunkedFile chunk = new FadvisedChunkedFile(spill,
@@ -1444,32 +1439,51 @@ public class ShuffleHandler extends AuxiliaryService {
       return writeFuture;
     }
 
-    protected void sendError(ChannelHandlerContext ctx,
-        HttpResponseStatus status) {
+    protected void sendError(ChannelHandlerContext ctx, HttpResponseStatus status) {
       sendError(ctx, "", status);
     }
 
-    protected void sendError(ChannelHandlerContext ctx, String message,
-        HttpResponseStatus status) {
-      HttpResponse response = new DefaultHttpResponse(HTTP_1_1, status);
-      response.setHeader(CONTENT_TYPE, "text/plain; charset=UTF-8");
+    protected void sendError(ChannelHandlerContext ctx, String message, HttpResponseStatus status) {
+      FullHttpResponse response = new DefaultFullHttpResponse(HTTP_1_1, status);
+      sendError(ctx, message, response);
+      response.release();
+    }
+
+    protected void sendError(ChannelHandlerContext ctx, String message, FullHttpResponse response) {
+      sendError(ctx, Unpooled.copiedBuffer(message, CharsetUtil.UTF_8), response);
+    }
+
+    private void sendFakeShuffleHeaderWithError(ChannelHandlerContext ctx, String message,
+        HttpResponse response) throws IOException {
+      FullHttpResponse fullResponse =
+          new DefaultFullHttpResponse(response.getProtocolVersion(), response.getStatus());
+      fullResponse.headers().set(response.headers());
+
+      ShuffleHeader header = new ShuffleHeader(message, -1, -1, -1);
+      DataOutputBuffer out = new DataOutputBuffer();
+      header.write(out);
+
+      sendError(ctx, wrappedBuffer(out.getData(), 0, out.getLength()), fullResponse);
+      fullResponse.release();
+    }
+
+    protected void sendError(ChannelHandlerContext ctx, ByteBuf content,
+        FullHttpResponse response) {
+      response.headers().set(CONTENT_TYPE, "text/plain; charset=UTF-8");
       // Put shuffle version into http header
-      response.setHeader(ShuffleHeader.HTTP_HEADER_NAME,
+      response.headers().set(ShuffleHeader.HTTP_HEADER_NAME,
           ShuffleHeader.DEFAULT_HTTP_HEADER_NAME);
-      response.setHeader(ShuffleHeader.HTTP_HEADER_VERSION,
+      response.headers().set(ShuffleHeader.HTTP_HEADER_VERSION,
           ShuffleHeader.DEFAULT_HTTP_HEADER_VERSION);
-      response.setContent(
-        ChannelBuffers.copiedBuffer(message, CharsetUtil.UTF_8));
+      response.content().writeBytes(content);
 
       // Close the connection as soon as the error message is sent.
-      ctx.getChannel().write(response).addListener(ChannelFutureListener.CLOSE);
+      ctx.channel().writeAndFlush(response).addListener(ChannelFutureListener.CLOSE);
     }
 
     @Override
-    public void exceptionCaught(ChannelHandlerContext ctx, ExceptionEvent e)
+    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause)
         throws Exception {
-      Channel ch = e.getChannel();
-      Throwable cause = e.getCause();
       if (cause instanceof TooLongFrameException) {
         sendError(ctx, BAD_REQUEST);
         return;
@@ -1486,8 +1500,8 @@ public class ShuffleHandler extends AuxiliaryService {
       }
 
       LOG.error("Shuffle error: ", cause);
-      if (ch.isConnected()) {
-        LOG.error("Shuffle error " + e);
+      if (ctx.channel().isActive()) {
+        LOG.error("Shuffle error", cause);
         sendError(ctx, INTERNAL_SERVER_ERROR);
       }
     }
