@@ -35,6 +35,7 @@ import java.util.concurrent.locks.ReentrantLock;
 
 import org.apache.commons.lang.exception.ExceptionUtils;
 import org.apache.hadoop.util.ShutdownHookManager;
+import org.apache.tez.common.GuavaShim;
 import org.apache.tez.common.TezTaskUmbilicalProtocol;
 import org.apache.tez.common.counters.TezCounters;
 import org.apache.tez.dag.api.TezException;
@@ -108,7 +109,7 @@ public class TaskReporter implements TaskReporterInterface {
     currentCallable = new HeartbeatCallable(task, umbilical, pollInterval, sendCounterInterval,
         maxEventsToGet, requestCounter, containerIdStr);
     ListenableFuture<Boolean> future = heartbeatExecutor.submit(currentCallable);
-    Futures.addCallback(future, new HeartbeatCallback(errorReporter));
+    Futures.addCallback(future, new HeartbeatCallback(errorReporter), GuavaShim.directExecutor());
   }
 
   /**
@@ -263,16 +264,12 @@ public class TaskReporter implements TaskReporterInterface {
       int maxEvents = Math.min(maxEventsToGet, task.getMaxEventsToHandle());
       TezHeartbeatRequest request = new TezHeartbeatRequest(requestId, events, fromPreRoutedEventId,
           containerIdStr, task.getTaskAttemptID(), fromEventId, maxEvents);
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Sending heartbeat to AM, request=" + request);
-      }
+      LOG.debug("Sending heartbeat to AM, request={}", request);
 
       maybeLogCounters();
 
       TezHeartbeatResponse response = umbilical.heartbeat(request);
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Received heartbeat response from AM, response=" + response);
-      }
+      LOG.debug("Received heartbeat response from AM, response={}", response);
 
       if (response.shouldDie()) {
         LOG.info("Received should die response from AM");
