@@ -29,7 +29,7 @@ import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.common.collect.Multimap;
 import org.apache.commons.lang.exception.ExceptionUtils;
 import org.apache.hadoop.conf.Configuration;
@@ -177,7 +177,8 @@ public class TezTaskRunner2 {
           // Safe to do this within a synchronized block because we're providing
           // the handler on which the Reporter will communicate back. Assuming
           // the register call doesn't end up hanging.
-          taskRunnerCallable = new TaskRunner2Callable(task, ugi);
+          taskRunnerCallable = new TaskRunner2Callable(task, ugi,
+              umbilicalAndErrorHandler);
           taskReporter.registerTask(task, umbilicalAndErrorHandler);
           future = executor.submit(taskRunnerCallable);
         }
