@@ -39,6 +39,7 @@ import org.slf4j.LoggerFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.LocalDirAllocator;
 import org.apache.hadoop.fs.Path;
+import org.apache.hadoop.fs.RawLocalFileSystem;
 import org.apache.hadoop.io.compress.CompressionCodec;
 import org.apache.tez.common.TezRuntimeFrameworkConfigs;
 import org.apache.tez.common.counters.TezCounter;
@@ -49,7 +50,9 @@ import org.apache.tez.runtime.library.common.shuffle.orderedgrouped.MapOutput.Ty
 import org.apache.tez.runtime.library.common.sort.impl.TezIndexRecord;
 import org.apache.tez.runtime.library.common.sort.impl.TezSpillRecord;
 import org.apache.tez.runtime.library.exceptions.FetcherReadTimeoutException;
+import org.apache.tez.runtime.library.common.shuffle.InputAttemptFetchFailure;
 import org.apache.tez.runtime.library.common.shuffle.ShuffleUtils;
+import org.apache.tez.runtime.library.common.shuffle.api.ShuffleHandlerError;
 
 import com.google.common.annotations.VisibleForTesting;
 
@@ -67,17 +70,17 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
   private final TezCounter ioErrs;
   private final TezCounter wrongLengthErrs;
   private final TezCounter badIdErrs;
-  private final TezCounter wrongMapErrs;
   private final TezCounter wrongReduceErrs;
   private final FetchedInputAllocatorOrderedGrouped allocator;
   private final ShuffleScheduler scheduler;
   private final ExceptionReporter exceptionReporter;
   private final int id;
   private final String logIdentifier;
+  private final RawLocalFileSystem localFs;
   private final String localShuffleHost;
   private final int localShufflePort;
   private final String applicationId;
- private final int dagId;
+  private final int dagId;
   private final MapHost mapHost;
 
   private final int minPartition;
@@ -114,6 +117,7 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
                                boolean ifileReadAhead, int ifileReadAheadLength,
                                CompressionCodec codec,
                                Configuration conf,
+                               RawLocalFileSystem localFs,
                                boolean localDiskFetchEnabled,
                                String localHostname,
                                int shufflePort,
@@ -143,7 +147,6 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
     this.ioErrs = ioErrsCounter;
     this.wrongLengthErrs = wrongLengthErrsCounter;
     this.badIdErrs = badIdErrsCounter;
-    this.wrongMapErrs = wrongMapErrsCounter;
     this.connectionErrs = connectionErrsCounter;
     this.wrongReduceErrs = wrongReduceErrsCounter;
     this.applicationId = applicationId;
@@ -159,6 +162,7 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
       this.codec = null;
     }
     this.conf = conf;
+    this.localFs = localFs;
     this.localShuffleHost = localHostname;
     this.localShufflePort = shufflePort;
 
@@ -204,9 +208,7 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
 
   public void shutDown() {
     if (!stopped) {
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Fetcher stopped for host " + mapHost);
-      }
+      LOG.debug("Fetcher stopped for host {}", mapHost);
       stopped = true;
       // An interrupt will come in while shutting down the thread.
       cleanupCurrentConnection(false);
@@ -270,7 +272,8 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
       // On any error, faildTasks is not null and we exit
       // after putting back the remaining maps to the 
       // yet_to_be_fetched list and marking the failed tasks.
-      InputAttemptIdentifier[] failedTasks = null;
+      InputAttemptFetchFailure[] failedTasks = null;
+
       while (!remaining.isEmpty() && failedTasks == null) {
         InputAttemptIdentifier inputAttemptIdentifier =
             remaining.entrySet().iterator().next().getValue();
@@ -283,40 +286,24 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
           // Setup connection again if disconnected
           cleanupCurrentConnection(true);
           if (stopped) {
-            if (LOG.isDebugEnabled()) {
-              LOG.debug("Not re-establishing connection since Fetcher has been stopped");
-            }
+            LOG.debug("Not re-establishing connection since Fetcher has been stopped");
             return;
           }
           // Connect with retry
           if (!setupConnection(host, remaining.values())) {
             if (stopped) {
               cleanupCurrentConnection(true);
-              if (LOG.isDebugEnabled()) {
-                LOG.debug(
-                    "Not reporting connection re-establishment failure since fetcher is stopped");
-              }
+              LOG.debug("Not reporting connection re-establishment failure since fetcher is stopped");
               return;
             }
-            failedTasks = new InputAttemptIdentifier[] {getNextRemainingAttempt()};
+            failedTasks = new InputAttemptFetchFailure[] {
+                new InputAttemptFetchFailure(getNextRemainingAttempt()) };
             break;
           }
         }
       }
 
-      if (failedTasks != null && failedTasks.length > 0) {
-        if (stopped) {
-          if (LOG.isDebugEnabled()) {
-            LOG.debug("Ignoring copyMapOutput failures for tasks: " + Arrays.toString(failedTasks) +
-                " since Fetcher has been stopped");
-          }
-        } else {
-          LOG.warn("copyMapOutput failed for tasks " + Arrays.toString(failedTasks));
-          for (InputAttemptIdentifier left : failedTasks) {
-            scheduler.copyFailed(left, host, true, false, false);
-          }
-        }
-      }
+      invokeCopyFailedForFailedTasks(host, failedTasks);
 
       cleanupCurrentConnection(false);
 
@@ -330,6 +317,23 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
     }
   }
 
+  private void invokeCopyFailedForFailedTasks(MapHost host,
+      InputAttemptFetchFailure[] failedTasks) {
+    if (failedTasks != null && failedTasks.length > 0) {
+      if (stopped) {
+        if (LOG.isDebugEnabled()) {
+          LOG.debug("Ignoring copyMapOutput failures for tasks: " + Arrays.toString(failedTasks)
+              + " since Fetcher has been stopped");
+        }
+      } else {
+        LOG.warn("copyMapOutput failed for tasks " + Arrays.toString(failedTasks));
+        for (InputAttemptFetchFailure left : failedTasks) {
+          scheduler.copyFailed(left, host, true, false);
+        }
+      }
+    }
+  }
+
   @VisibleForTesting
   boolean setupConnection(MapHost host, Collection<InputAttemptIdentifier> attempts)
       throws IOException {
@@ -343,9 +347,7 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
       connectSucceeded = httpConnection.connect();
 
       if (stopped) {
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Detected fetcher has been shutdown after connection establishment. Returning");
-        }
+        LOG.debug("Detected fetcher has been shutdown after connection establishment. Returning");
         return false;
       }
       input = httpConnection.getInputStream();
@@ -356,18 +358,18 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
         Thread.currentThread().interrupt(); //reset status
       }
       if (stopped) {
-        if (LOG.isDebugEnabled()) {
-          LOG.debug("Not reporting fetch failure, since an Exception was caught after shutdown");
-        }
+        LOG.debug("Not reporting fetch failure, since an Exception was caught after shutdown");
         return false;
       }
       ioErrs.increment(1);
       if (!connectSucceeded) {
-        LOG.warn("Failed to connect to " + host + " with " + remaining.size() + " inputs", ie);
+        LOG.warn(String.format("Failed to connect from %s to %s with %d inputs", localShuffleHost,
+            host, remaining.size()), ie);
         connectionErrs.increment(1);
       } else {
-        LOG.warn("Failed to verify reply after connecting to " + host + " with " + remaining.size()
-            + " inputs pending", ie);
+        LOG.warn(String.format(
+            "Failed to verify reply after connecting from %s to %s with %d inputs pending",
+            localShuffleHost, host, remaining.size()), ie);
       }
 
       // At this point, either the connection failed, or the initial header verification failed.
@@ -376,7 +378,8 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
       for (InputAttemptIdentifier left : remaining.values()) {
         // Need to be handling temporary glitches ..
         // Report read error to the AM to trigger source failure heuristics
-        scheduler.copyFailed(left, host, connectSucceeded, !connectSucceeded, false);
+        scheduler.copyFailed(InputAttemptFetchFailure.fromAttempt(left), host, connectSucceeded,
+            !connectSucceeded);
       }
       return false;
     }
@@ -400,7 +403,8 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
     }
   }
 
-  private static InputAttemptIdentifier[] EMPTY_ATTEMPT_ID_ARRAY = new InputAttemptIdentifier[0];
+  private static final InputAttemptFetchFailure[] EMPTY_ATTEMPT_ID_ARRAY =
+      new InputAttemptFetchFailure[0];
 
   private static class MapOutputStat {
     final InputAttemptIdentifier srcAttemptId;
@@ -421,8 +425,8 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
     }
   }
 
-  protected InputAttemptIdentifier[] copyMapOutput(MapHost host,
-                                DataInputStream input, InputAttemptIdentifier inputAttemptIdentifier) throws FetcherReadTimeoutException {
+  protected InputAttemptFetchFailure[] copyMapOutput(MapHost host, DataInputStream input,
+      InputAttemptIdentifier inputAttemptIdentifier) throws FetcherReadTimeoutException {
     MapOutput mapOutput = null;
     InputAttemptIdentifier srcAttemptId = null;
     long decompressedLength = 0;
@@ -448,11 +452,15 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
               badIdErrs.increment(1);
               LOG.warn("Invalid map id: " + header.mapId + ", expected to start with " +
                   InputAttemptIdentifier.PATH_PREFIX + ", partition: " + header.forReduce);
-              return new InputAttemptIdentifier[]{getNextRemainingAttempt()};
-            } else {
-              if (LOG.isDebugEnabled()) {
-                LOG.debug("Already shutdown. Ignoring invalid map id error");
+              if (header.mapId.startsWith(ShuffleHandlerError.DISK_ERROR_EXCEPTION.toString())) {
+                //this should be treated as local fetch failure while reporting later
+                return new InputAttemptFetchFailure[] {
+                    InputAttemptFetchFailure.fromDiskErrorAtSource(getNextRemainingAttempt()) };
               }
+              return new InputAttemptFetchFailure[] {
+                  InputAttemptFetchFailure.fromAttempt(getNextRemainingAttempt()) };
+            } else {
+              LOG.debug("Already shutdown. Ignoring invalid map id error");
               return EMPTY_ATTEMPT_ID_ARRAY;
             }
           }
@@ -473,7 +481,8 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
             LOG.warn("Invalid map id ", e);
             // Don't know which one was bad, so consider this one bad and dont read
             // the remaining because we dont know where to start reading from. YARN-1773
-            return new InputAttemptIdentifier[]{getNextRemainingAttempt()};
+            return new InputAttemptFetchFailure[] {
+                new InputAttemptFetchFailure(getNextRemainingAttempt()) };
           } else {
             if (LOG.isDebugEnabled()) {
               LOG.debug("Already shutdown. Ignoring invalid map id error. Exception: " +
@@ -493,11 +502,10 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
               LOG.warn("Was expecting " + srcAttemptId + " but got null");
             }
             assert (srcAttemptId != null);
-            return new InputAttemptIdentifier[]{srcAttemptId};
+            return new InputAttemptFetchFailure[] {
+                new InputAttemptFetchFailure(getNextRemainingAttempt()) };
           } else {
-            if (LOG.isDebugEnabled()) {
-              LOG.debug("Already stopped. Ignoring verification failure.");
-            }
+            LOG.debug("Already stopped. Ignoring verification failure.");
             return EMPTY_ATTEMPT_ID_ARRAY;
           }
         }
@@ -521,9 +529,7 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
             ioErrs.increment(1);
             scheduler.reportLocalError(e);
           } else {
-            if (LOG.isDebugEnabled()) {
-              LOG.debug("Already stopped. Ignoring error from merger.reserve");
-            }
+            LOG.debug("Already stopped. Ignoring error from merger.reserve");
           }
           return EMPTY_ATTEMPT_ID_ARRAY;
         }
@@ -591,9 +597,10 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
             srcAttemptId + " decomp: " +
             decompressedLength + ", " + compressedLength, ioe);
         if (srcAttemptId == null) {
-          return remaining.values().toArray(new InputAttemptIdentifier[remaining.values().size()]);
+          return InputAttemptFetchFailure.fromAttempts(remaining.values());
         } else {
-          return new InputAttemptIdentifier[]{srcAttemptId};
+          return new InputAttemptFetchFailure[] {
+              new InputAttemptFetchFailure(srcAttemptId) };
         }
       }
       LOG.warn("Failed to shuffle output of " + srcAttemptId +
@@ -601,7 +608,8 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
 
       // Inform the shuffle-scheduler
       mapOutput.abort();
-      return new InputAttemptIdentifier[] {srcAttemptId};
+      return new InputAttemptFetchFailure[] {
+          new InputAttemptFetchFailure(srcAttemptId) };
     }
     return null;
   }
@@ -730,14 +738,12 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
             if (!stopped) {
               hasFailures = true;
               ioErrs.increment(1);
-              scheduler.copyFailed(srcAttemptId, host, true, false, true);
+              scheduler.copyFailed(InputAttemptFetchFailure.fromLocalFetchFailure(srcAttemptId),
+                  host, true, false);
               LOG.warn("Failed to read local disk output of " + srcAttemptId + " from " +
                   host.getHostIdentifier(), e);
             } else {
-              if (LOG.isDebugEnabled()) {
-                LOG.debug(
-                    "Ignoring fetch error during local disk copy since fetcher has already been stopped");
-              }
+              LOG.debug("Ignoring fetch error during local disk copy since fetcher has already been stopped");
               return;
             }
 
@@ -772,7 +778,7 @@ class FetcherOrderedGrouped extends CallableWithNdc<Void> {
       throws IOException {
     Path indexFile = getShuffleInputFileName(pathComponent,
         Constants.TEZ_RUNTIME_TASK_OUTPUT_INDEX_SUFFIX_STRING);
-    TezSpillRecord spillRecord = new TezSpillRecord(indexFile, conf);
+    TezSpillRecord spillRecord = new TezSpillRecord(indexFile, localFs);
     return spillRecord.getIndex(partitionId);
   }
 
