@@ -18,7 +18,7 @@
 package org.apache.tez.runtime.library.common.shuffle.orderedgrouped;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import org.apache.commons.io.FilenameUtils;
 import org.apache.hadoop.classification.InterfaceAudience;
 import org.apache.hadoop.classification.InterfaceAudience.Private;
@@ -30,7 +30,6 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.LocalDirAllocator;
 import org.apache.hadoop.fs.LocalFileSystem;
 import org.apache.hadoop.fs.Path;
-import org.apache.hadoop.io.BoundedByteArrayOutputStream;
 import org.apache.hadoop.io.DataInputBuffer;
 import org.apache.hadoop.io.FileChunk;
 import org.apache.hadoop.io.RawComparator;
@@ -47,6 +46,7 @@ import org.apache.tez.runtime.library.common.ConfigUtils;
 import org.apache.tez.runtime.library.common.Constants;
 import org.apache.tez.runtime.library.common.InputAttemptIdentifier;
 import org.apache.tez.runtime.library.common.combine.Combiner;
+import org.apache.tez.runtime.library.common.serializer.SerializationContext;
 import org.apache.tez.runtime.library.common.sort.impl.IFile;
 import org.apache.tez.runtime.library.common.sort.impl.IFile.Writer;
 import org.apache.tez.runtime.library.common.sort.impl.TezMerger;
@@ -158,6 +158,8 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
 
   private final boolean cleanup;
 
+  private SerializationContext serializationContext;
+
   /**
    * Construct the MergeManager. Must call start before it becomes usable.
    */
@@ -297,21 +299,31 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
           + ", mergeThreshold: " + this.mergeThreshold);
     }
     
-    boolean allowMemToMemMerge = 
-        conf.getBoolean(
-            TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_ENABLE_MEMTOMEM, 
+    boolean allowMemToMemMerge =
+        conf.getBoolean(TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_ENABLE_MEMTOMEM,
             TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_ENABLE_MEMTOMEM_DEFAULT);
-      if (allowMemToMemMerge) {
-        this.memToMemMerger = 
-          new IntermediateMemoryToMemoryMerger(this,
-                                               memToMemMergeOutputsThreshold);
-      } else {
-        this.memToMemMerger = null;
-      }
-      
-      this.inMemoryMerger = new InMemoryMerger(this);
-      
-      this.onDiskMerger = new OnDiskMerger(this);
+    if (allowMemToMemMerge) {
+      this.memToMemMerger =
+          new IntermediateMemoryToMemoryMerger(this, memToMemMergeOutputsThreshold);
+    } else {
+      this.memToMemMerger = null;
+    }
+
+    this.inMemoryMerger = new InMemoryMerger(this);
+
+    this.onDiskMerger = new OnDiskMerger(this);
+
+    this.serializationContext = new SerializationContext(conf);
+  }
+
+  void setupParentThread(Thread shuffleSchedulerThread) {
+    LOG.info("Setting merger's parent thread to "
+        + shuffleSchedulerThread.getName());
+    if (this.memToMemMerger != null) {
+      memToMemMerger.setParentThread(shuffleSchedulerThread);
+    }
+    this.inMemoryMerger.setParentThread(shuffleSchedulerThread);;
+    this.onDiskMerger.setParentThread(shuffleSchedulerThread);
   }
 
   @Private
@@ -376,9 +388,7 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
     }
     if (triggerAdditionalMerge) {
       inMemoryMerger.waitForMerge();
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Additional in-memory merge triggered");
-      }
+      LOG.debug("Additional in-memory merge triggered");
     }
   }
 
@@ -688,9 +698,7 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
     }
 
     try {
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Deleting " + path);
-      }
+      LOG.debug("Deleting {}", path);
       fs.delete(path, true);
     } catch (IOException e) {
       LOG.info("Error in deleting " + path);
@@ -714,7 +722,8 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
                                             int mergeFactor) {
       super(manager, mergeFactor, exceptionReporter);
       setName("MemToMemMerger [" + TezUtilsInternal
-          .cleanVertexName(inputContext.getSourceVertexName()) + "]");
+          .cleanVertexName(inputContext.getSourceVertexName())
+          + "_" + inputContext.getUniqueIdentifier() + "]");
       setDaemon(true);
     }
 
@@ -791,8 +800,7 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
       // TODO Is this doing any combination ?
       TezRawKeyValueIterator rIter = 
         TezMerger.merge(conf, rfs,
-                       ConfigUtils.getIntermediateInputKeyClass(conf),
-                       ConfigUtils.getIntermediateInputValueClass(conf),
+                       serializationContext,
                        inMemorySegments, inMemorySegments.size(),
                        new Path(inputContext.getUniqueIdentifier()),
                        (RawComparator)ConfigUtils.getIntermediateInputKeyComparator(conf),
@@ -831,8 +839,9 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
 
     public InMemoryMerger(MergeManager manager) {
       super(manager, Integer.MAX_VALUE, exceptionReporter);
-      setName("MemtoDiskMerger [" + TezUtilsInternal
-          .cleanVertexName(inputContext.getSourceVertexName()) + "]");
+      setName("MemtoDiskMerger [" +  TezUtilsInternal
+          .cleanVertexName(inputContext.getSourceVertexName())
+          + "_" + inputContext.getUniqueIdentifier()  + "]");
       setDaemon(true);
     }
     
@@ -873,10 +882,10 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
       long outFileLen = 0;
       try {
         writer =
-            new Writer(conf, rfs, outputPath,
-                (Class)ConfigUtils.getIntermediateInputKeyClass(conf),
-                (Class)ConfigUtils.getIntermediateInputValueClass(conf),
-                codec, null, null);
+            new Writer(serializationContext.getKeySerialization(),
+                serializationContext.getValSerialization(), rfs, outputPath,
+                serializationContext.getKeyClass(), serializationContext.getValueClass(), codec,
+                null, null);
 
         TezRawKeyValueIterator rIter = null;
         LOG.info("Initiating in-memory merge with " + noInMemorySegments + 
@@ -885,8 +894,7 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
         tmpDir = new Path(inputContext.getUniqueIdentifier());
         // Nothing actually materialized to disk - controlled by setting sort-factor to #segments.
         rIter = TezMerger.merge(conf, rfs,
-            (Class)ConfigUtils.getIntermediateInputKeyClass(conf),
-            (Class)ConfigUtils.getIntermediateInputValueClass(conf),
+            serializationContext,
             inMemorySegments, inMemorySegments.size(),
             tmpDir, (RawComparator)ConfigUtils.getIntermediateInputKeyComparator(conf),
             progressable, spilledRecordsCounter, null, additionalBytesRead, null);
@@ -952,8 +960,9 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
 
     public OnDiskMerger(MergeManager manager) {
       super(manager, ioSortFactor, exceptionReporter);
-      setName("DiskToDiskMerger [" + TezUtilsInternal
-          .cleanVertexName(inputContext.getSourceVertexName()) + "]");
+      setName("DiskToDiskMerger [" +  TezUtilsInternal
+          .cleanVertexName(inputContext.getSourceVertexName())
+          + "_" + inputContext.getUniqueIdentifier() + "]");
       setDaemon(true);
     }
     
@@ -1014,16 +1023,14 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
       outputPath = localDirAllocator.getLocalPathForWrite(namePart, approxOutputSize, conf);
       outputPath = outputPath.suffix(Constants.MERGED_OUTPUT_PREFIX + mergeFileSequenceId.getAndIncrement());
 
-      Writer writer =
-        new Writer(conf, rfs, outputPath, 
-                        (Class)ConfigUtils.getIntermediateInputKeyClass(conf), 
-                        (Class)ConfigUtils.getIntermediateInputValueClass(conf),
-                        codec, null, null);
+      Writer writer = new Writer(serializationContext.getKeySerialization(),
+          serializationContext.getValSerialization(), rfs, outputPath,
+          serializationContext.getKeyClass(), serializationContext.getValueClass(), codec, null,
+          null);
       tmpDir = new Path(inputContext.getUniqueIdentifier());
       try {
         TezRawKeyValueIterator iter = TezMerger.merge(conf, rfs,
-            (Class)ConfigUtils.getIntermediateInputKeyClass(conf),
-            (Class)ConfigUtils.getIntermediateInputValueClass(conf),
+            serializationContext,
             inputSegments,
             ioSortFactor, tmpDir,
             (RawComparator)ConfigUtils.getIntermediateInputKeyComparator(conf),
@@ -1117,7 +1124,7 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
         final int klen = kb.getLength() - kp;
         key.reset(kb.getData(), kp, klen);
         bytesRead += klen;
-        return KeyState.NEW_KEY;
+        return kvIter.isSameKey() ? KeyState.SAME_KEY : KeyState.NEW_KEY;
       }
       return KeyState.NO_KEY;
     }
@@ -1152,8 +1159,7 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
     inputContext.notifyProgress();
 
     // merge config params
-    Class keyClass = (Class)ConfigUtils.getIntermediateInputKeyClass(job);
-    Class valueClass = (Class)ConfigUtils.getIntermediateInputValueClass(job);
+    SerializationContext serContext = new SerializationContext(job);
     final Path tmpDir = new Path(inputContext.getUniqueIdentifier());
     final RawComparator comparator =
       (RawComparator)ConfigUtils.getIntermediateInputKeyComparator(job);
@@ -1185,11 +1191,12 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
         final Path outputPath = 
           mapOutputFile.getInputFileForWrite(srcTaskId, Integer.MAX_VALUE,
               inMemToDiskBytes).suffix(Constants.MERGED_OUTPUT_PREFIX);
-        final TezRawKeyValueIterator rIter = TezMerger.merge(job, fs, keyClass, valueClass,
+        final TezRawKeyValueIterator rIter = TezMerger.merge(job, fs, serContext,
             memDiskSegments, numMemDiskSegments, tmpDir, comparator, progressable,
             spilledRecordsCounter, null, additionalBytesRead, null);
-        final Writer writer = new Writer(job, fs, outputPath,
-            keyClass, valueClass, codec, null, null);
+        final Writer writer = new Writer(serContext.getKeySerialization(),
+            serContext.getValSerialization(), fs, outputPath, serContext.getKeyClass(),
+            serContext.getValueClass(), codec, null, null);
         try {
           TezMerger.writeFile(rIter, writer, progressable, TezRuntimeConfiguration.TEZ_RUNTIME_RECORDS_BEFORE_PROGRESS_DEFAULT);
         } catch (IOException e) {
@@ -1289,7 +1296,7 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
       diskSegments.addAll(0, memDiskSegments);
       memDiskSegments.clear();
       TezRawKeyValueIterator diskMerge = TezMerger.merge(
-          job, fs, keyClass, valueClass, codec, diskSegments,
+          job, fs, serContext, codec, diskSegments,
           ioSortFactor, numInMemSegments, tmpDir, comparator,
           progressable, false, spilledRecordsCounter, null, additionalBytesRead, null);
       diskSegments.clear();
@@ -1303,10 +1310,10 @@ public class MergeManager implements FetchedInputAllocatorOrderedGrouped {
       LOG.info(finalMergeLog.toString());
     }
     // This is doing nothing but creating an iterator over the segments.
-    return TezMerger.merge(job, fs, keyClass, valueClass,
-                 finalSegments, finalSegments.size(), tmpDir,
-                 comparator, progressable, spilledRecordsCounter, null,
-                 additionalBytesRead, null);
+    return TezMerger.merge(job, fs, serContext, codec,
+        finalSegments, finalSegments.size(), tmpDir,
+        comparator, progressable, spilledRecordsCounter, null,
+        additionalBytesRead, null);
   }
 
 
