@@ -24,9 +24,10 @@ import javax.annotation.Nullable;
 
 import java.io.IOException;
 import java.util.Map;
+import java.util.Objects;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.common.collect.Lists;
 
 import org.apache.hadoop.classification.InterfaceAudience;
@@ -299,8 +300,8 @@ public class OrderedGroupedKVInputConfig {
     @InterfaceAudience.Private
     Builder(String keyClassName, String valueClassName) {
       this();
-      Preconditions.checkNotNull(keyClassName, "Key class name cannot be null");
-      Preconditions.checkNotNull(valueClassName, "Value class name cannot be null");
+      Objects.requireNonNull(keyClassName, "Key class name cannot be null");
+      Objects.requireNonNull(valueClassName, "Value class name cannot be null");
       setKeyClassName(keyClassName);
       setValueClassName(valueClassName);
     }
@@ -316,14 +317,14 @@ public class OrderedGroupedKVInputConfig {
 
     @InterfaceAudience.Private
     Builder setKeyClassName(String keyClassName) {
-      Preconditions.checkNotNull(keyClassName, "Key class name cannot be null");
+      Objects.requireNonNull(keyClassName, "Key class name cannot be null");
       this.conf.set(TezRuntimeConfiguration.TEZ_RUNTIME_KEY_CLASS, keyClassName);
       return this;
     }
 
     @InterfaceAudience.Private
     Builder setValueClassName(String valueClassName) {
-      Preconditions.checkNotNull(valueClassName, "Value class name cannot be null");
+      Objects.requireNonNull(valueClassName, "Value class name cannot be null");
       this.conf.set(TezRuntimeConfiguration.TEZ_RUNTIME_VALUE_CLASS, valueClassName);
       return this;
     }
@@ -404,7 +405,7 @@ public class OrderedGroupedKVInputConfig {
      */
     public Builder setKeyComparatorClass(String comparatorClassName,
                                          @Nullable Map<String, String> comparatorConf) {
-      Preconditions.checkNotNull(comparatorClassName, "Comparator class name cannot be null");
+      Objects.requireNonNull(comparatorClassName, "Comparator class name cannot be null");
       this.conf.set(TezRuntimeConfiguration.TEZ_RUNTIME_KEY_COMPARATOR_CLASS,
           comparatorClassName);
       if (comparatorConf != null) {
@@ -418,7 +419,7 @@ public class OrderedGroupedKVInputConfig {
     @SuppressWarnings("unchecked")
     @Override
     public Builder setAdditionalConfiguration(String key, String value) {
-      Preconditions.checkNotNull(key, "Key cannot be null");
+      Objects.requireNonNull(key, "Key cannot be null");
       if (ConfigUtils.doesKeyQualify(key,
           Lists.newArrayList(OrderedGroupedKVInput.getConfigurationKeySet(),
               TezRuntimeConfiguration.getRuntimeAdditionalConfigKeySet()),
@@ -435,7 +436,7 @@ public class OrderedGroupedKVInputConfig {
     @SuppressWarnings("unchecked")
     @Override
     public Builder setAdditionalConfiguration(Map<String, String> confMap) {
-      Preconditions.checkNotNull(confMap, "ConfMap cannot be null");
+      Objects.requireNonNull(confMap, "ConfMap cannot be null");
       Map<String, String> map = ConfigUtils.extractConfigurationMap(confMap,
           Lists.newArrayList(OrderedGroupedKVInput.getConfigurationKeySet(),
               TezRuntimeConfiguration.getRuntimeAdditionalConfigKeySet()), TezRuntimeConfiguration.getAllowedPrefixes());
