@@ -18,7 +18,7 @@
 
 package org.apache.tez.runtime.library.common.sort.impl;
 
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.common.collect.LinkedListMultimap;
 import com.google.common.collect.ListMultimap;
 import com.google.common.collect.Lists;
@@ -36,11 +36,13 @@ import org.apache.hadoop.io.LongWritable;
 import org.apache.hadoop.io.RawComparator;
 import org.apache.hadoop.io.Text;
 import org.apache.hadoop.io.WritableComparator;
+import org.apache.hadoop.io.serializer.WritableSerialization;
 import org.apache.hadoop.util.Progress;
 import org.apache.hadoop.util.Progressable;
 import org.apache.tez.common.TezRuntimeFrameworkConfigs;
 import org.apache.tez.runtime.library.api.TezRuntimeConfiguration;
 import org.apache.tez.runtime.library.common.ConfigUtils;
+import org.apache.tez.runtime.library.common.serializer.SerializationContext;
 import org.apache.tez.runtime.library.common.shuffle.orderedgrouped.InMemoryReader;
 import org.apache.tez.runtime.library.common.shuffle.orderedgrouped.InMemoryWriter;
 import org.apache.tez.runtime.library.common.shuffle.orderedgrouped.MergeManager;
@@ -136,7 +138,7 @@ public class TestTezMerger {
   private Path createIFileWithTextData(List<String> data) throws IOException {
     Path path = new Path(workDir + "/src", "data_" + System.nanoTime() + ".out");
     FSDataOutputStream out = localFs.create(path);
-    IFile.Writer writer = new IFile.Writer(defaultConf, out, Text.class,
+    IFile.Writer writer = new IFile.Writer(new WritableSerialization(), new WritableSerialization(), out, Text.class,
         Text.class, null, null, null, true);
     for (String key : data) {
       writer.append(new Text(key), new Text(key + "_" + System.nanoTime()));
@@ -568,11 +570,12 @@ public class TestTezMerger {
   private TezRawKeyValueIterator merge(List<Path> pathList, RawComparator rc)
       throws IOException, InterruptedException {
     TezMerger merger = new TezMerger();
-    TezRawKeyValueIterator records = merger.merge(defaultConf, localFs, IntWritable.class,
-        LongWritable.class, null, false, 0, 1024, pathList.toArray(new Path[pathList.size()]),
-        true, 4, new Path(workDir, "tmp_" + System.nanoTime()), ((rc == null) ? comparator : rc),
-        new Reporter(), null, null,
-        null, new Progress());
+    TezRawKeyValueIterator records = merger.merge(defaultConf, localFs,
+        new SerializationContext(IntWritable.class, LongWritable.class, new WritableSerialization(),
+            new WritableSerialization()),
+        null, false, 0, 1024, pathList.toArray(new Path[pathList.size()]), true, 4,
+        new Path(workDir, "tmp_" + System.nanoTime()), ((rc == null) ? comparator : rc),
+        new Reporter(), null, null, null, new Progress());
     return records;
   }
 
@@ -602,12 +605,12 @@ public class TestTezMerger {
   private void merge(List<Path> pathList, int mergeFactor, RawComparator rc) throws Exception {
     //Merge datasets
     TezMerger merger = new TezMerger();
-    TezRawKeyValueIterator records = merger.merge(defaultConf, localFs, IntWritable.class,
-        LongWritable.class, null, false, 0, 1024, pathList.toArray(new Path[pathList.size()]),
-        true, mergeFactor, new Path(workDir, "tmp_" + System.nanoTime()),
-        ((rc == null) ? comparator : rc), new Reporter(), null, null,
-        null,
-        new Progress());
+    TezRawKeyValueIterator records = merger.merge(defaultConf, localFs,
+        new SerializationContext(IntWritable.class, LongWritable.class, new WritableSerialization(),
+            new WritableSerialization()),
+        null, false, 0, 1024, pathList.toArray(new Path[pathList.size()]), true, mergeFactor,
+        new Path(workDir, "tmp_" + System.nanoTime()), ((rc == null) ? comparator : rc),
+        new Reporter(), null, null, null, new Progress());
 
     verifyData(records);
     verificationDataSet.clear();
@@ -701,9 +704,10 @@ public class TestTezMerger {
     TezMerger.MergeQueue mergeQueue = new TezMerger.MergeQueue(defaultConf, localFs, segmentList,
         comparator, new Reporter(), false, false);
 
-    TezRawKeyValueIterator records = mergeQueue.merge(IntWritable.class, LongWritable.class,
-        mergeFactor, new Path(workDir, "tmp_"
-        + System.nanoTime()), null, null, null, null);
+    TezRawKeyValueIterator records = mergeQueue.merge(
+        new SerializationContext(IntWritable.class, LongWritable.class, new WritableSerialization(),
+            new WritableSerialization()),
+        mergeFactor, new Path(workDir, "tmp_" + System.nanoTime()), null, null, null, null);
 
     //Verify the merged data is correct
     verifyData(records);
@@ -770,8 +774,8 @@ public class TestTezMerger {
     Path path = new Path(workDir + "/src", "data_" + System.nanoTime() + ".out");
     FSDataOutputStream out = localFs.create(path);
     //create IFile with RLE
-    IFile.Writer writer = new IFile.Writer(defaultConf, out, IntWritable.class
-        , LongWritable.class, null, null, null, true);
+    IFile.Writer writer = new IFile.Writer(new WritableSerialization(), new WritableSerialization(),
+        out, IntWritable.class, LongWritable.class, null, null, null, true);
 
     for (Integer key : dataSet.keySet()) {
       for (Long value : dataSet.get(key)) {
