@@ -43,7 +43,7 @@ import org.apache.tez.history.parser.datamodel.TaskAttemptInfo.DataDependencyEve
 import org.apache.tez.history.parser.datamodel.TaskInfo;
 
 import com.google.common.base.Joiner;
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.common.base.Strings;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
